import java.math.*;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.*;
import com.justep.system.data.Table;
import javax.swing.text.html.HTMLDocument.HTMLReader.IsindexAction;
import java.util.Calendar;
import com.tool;
import org.dom4j.*;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.justep.system.data.*;
import com.sun.glass.ui.Pixels.Format;
import com.Sql;

public class Manufacturing {

	private static final String DATA_MODEL = "/DB/table/data";// 表的模块
	private static SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
	private static int eIndex = 0;// 设备数组的下标
	private static int StandardCapacity = 0;// 工作中心标准产能
	private static int MaximumCapacity = 0;// 工作中心最大产能

	/**
	 * 
	 * @Title: discharge
	 * @Description: 生成热线排产计划(主方法)
	 * @param date
	 *            计算哪天的排产计划
	 * @author ChenXiaoChen
	 * @date 2018-11-9
	 * @return void 返回类型
	 * 
	 */
	public static void discharge(Date date) {
		/**
		 * 生成当前日期排程计划 1.排产时，首先排产非熔炼工作中心（排产模具优先的工作中心，排产时，需要考虑模具的个数，以及产线的生产能力）
		 * （铁水优先的工作中心，排产时，需要配比铁水的材质，熔炼量等） 由于电炉资源是公用的，所以需要考虑到电炉的熔炼量
		 * 2.其次，熔炼的工作中心根据其余热线任务，进行排产
		 */
		try {

			// 建立电炉对象，作为公共资源
			List<Equipment> list = new ArrayList<Equipment>();// 用于存放熔炼设备的list
			List<Task> taskList = new ArrayList<Task>();// 存放当天的任务对象
			String sql = "select a.* from AS_Ledger a ";
			Table equipmentTable = SQL.select(Sql.DBMySql(sql), null, DATA_MODEL);
			Iterator<Row> equipmentrows = equipmentTable.iterator();
			while (equipmentrows.hasNext()) {
				Row row = equipmentrows.next();
				String ID = row.getString("fID");// 设备ID
				String code = row.getString("fCode");// 编码
				String name = row.getString("fName");// 设备名称
				int meltingCapacity = row.getInt("fMeltingCapacity");// 熔炼量
				int maxTime = row.getInt("fMaxTime");//最大开炉数
				int MinMeltingCapacity = row.getInt("fMinMeltingCapacity");//最小熔炼量
				Equipment e = new Equipment(ID, code, name, meltingCapacity, maxTime,MinMeltingCapacity);
				list.add(e);
			}

			System.out.println("开始排程" + format.format(date));
			// 安排非熔炼工作中心的任务,当前的任务
			sql = "select a.* from BAS_WorkCenter a where a.fColdOrHot = '热线' and a.fIsMelting = '否'";
			Table table = SQL.select(Sql.DBMySql(sql), null, DATA_MODEL);
			Iterator<Row> rows = table.iterator();
			while (rows.hasNext()) {
				Row row = rows.next();
				String type = row.getString("fSchedulingPriority");// 排产方式
				StandardCapacity = row.getInt("fStandardCapacity");// 标准产能
				MaximumCapacity = row.getInt("fMaximumCapacity");// 最大产能
				// if("模具优先".equals(type)){
				// int maxNum = row.getInt("fMaxMouldNum");//最大模具数
				moldPriority(row.getString("fID"), 22, list, date, taskList);
				// }else if("铁水优先".equals(type)){
				// HotMetalPriority(row.getString("fID"));
				// }
			}

			// 所有线派完后，生成熔炼的开炉单
			sql = "select a.* from BAS_WorkCenter a where a.fColdOrHot = '热线' and a.fIsMelting = '是'";
			Table melting = SQL.select(Sql.DBMySql(sql), null, DATA_MODEL);
			String workCenterID = melting.iterator().next().getString("fID");
			String ID = UUID.randomUUID().toString().replace("-", "");// 生成32位字符串
			// 生成派工单
			String ksql = "insert into MES_DispatchedWorkers a (a,a.version,a.fDocumentNumber,a.fStatus,a.fCase,a.fWorkCenterID,a.fDate) " + "values('" + ID
					+ "',0,:nextSeqString(format('%s%s','PG',format('%1$tY%1$tm%1$td', currentDateTime())),'0000'),'未确认','计划优先','" + workCenterID + "',stringToDate('" + format.format(date) + "'))";
			KSQL.executeUpdate(ksql, null, DATA_MODEL, null);
			for (int i = 0; i < taskList.size(); i++) {
				Task t = taskList.get(i);
				sql = "insert into MES_DispatchedWorkersList (fID,version,fMainID,fProcedurePlanID,fNumber,fResourceID,fWeight,fTotalHours) " + "values(replace(uuid(), '-', ''),0,'" + ID + "','"
						+ t.getID() + "'," + t.getNum() + ",'" + t.getDeviceID() + "'," + t.getWeight() + "," + t.getHours() + ")";
				SQL.executeUpdate(Sql.DBMySql(sql), null, DATA_MODEL);
			}

			// return "ok";
		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException(e.getMessage());
		}
	}

	// 模具优先
	public static void moldPriority(String workerCenterID, int maxMouldNum, List<Equipment> list, Date date, List<Task> taskList) {
		/*
		 * 1.找出本工作中心的电炉资源 2.计算出本道工序所需要的铁水量(包括共模订单的重量) 3.凑铁水 4.确认派工数量
		 */

		String ID = UUID.randomUUID().toString().replace("-", "");// 生成32位字符串
		// 生成派工单
		String ksql = "insert into MES_DispatchedWorkers a (a,a.version,a.fDocumentNumber,a.fStatus,a.fCase,a.fWorkCenterID,a.fDate) " + "values('" + ID
				+ "',0,:nextSeqString(format('%s%s','PG',format('%1$tY%1$tm%1$td', currentDateTime())),'0000'),'未确认','计划优先','" + workerCenterID + "',stringToDate('" + format.format(date) + "'))";
		KSQL.executeUpdate(ksql, null, DATA_MODEL, null);
		String[] Equipment = null;
		// 找出与之相关的电炉
		String sql = "select a.* from MES_ElectricFurnace a where a.fWorkCenterID = '" + workerCenterID + "' order by a.fPriorityRight,a.fPriority asc ";
		Table EquipmentTable = SQL.select(Sql.DBMySql(sql), null, DATA_MODEL);
		if (EquipmentTable.size() > 0) {
			Equipment = new String[EquipmentTable.size()];
			Iterator<Row> rows = EquipmentTable.iterator();
			int i = 0;
			while (rows.hasNext()) {
				Row row = rows.next();
				Equipment[i] = row.getString("fEquipmentID");
				i++;
			}
		} else {
			return;
		}

		eIndex = 0;

		//查出需要派工的记录
		sql = "select a.fID,a.fTotalHours,a.fFinishNumber,ifnull(e.fOutputRate,1) as fOutputRate,g.fRationNum as RationNum ,b.fMouldCode,b.fCommonModeSign as fCommonModeSign,b.fTaskBookID as fTaskBookID,b.fStockCode,e.fMaterialType,b.fNumber,sum( f.fNumber ) AS pNumber " +
				"from MES_ProcedurePlan a " +
				"left join MES_ProductionOrder b on b.fID = a.fMainID " +
				"left join BAS_StockArchives e on e.fStockCode = b.fStockCode " +
				"left join  MES_DispatchedWorkersList f ON f.fProcedurePlanID = a.fID " +
				"left join (SELECT	c.fRationNum,c.fStockCode  FROM	CAPP_MouldProductList c	LEFT JOIN CAPP_Mould d ON d.fID = c.fMainID WHERE d.fStatus = '有效') g ON  g.fStockCode = b.fStockCode " +
				" where a.fWorkCenterID = '"+workerCenterID+"' and a.fIsContinuity = '是'  and b.fOrderStatus = '生产' and a.fPlannedStartDate is not null " +
				" group by a.fID,a.fFinishNumber,b.fCommonModeSign,b.fTaskBookID,b.fStockCode,e.fMaterialType,b.fNumber,g.fRationNum,a.fTotalHours,b.fMouldCode,e.fOutputRate " +
				"having b.fNumber > ifnull( sum( f.fNumber ), 0 ) " +
				"order by a.fPlannedStartDate,b.fRatio  ";//    and a.fIsFinish = '否'

		Table table = SQL.select(Sql.DBMySql(sql), null, DATA_MODEL);
		Iterator<Row> rows = table.iterator();
		while (rows.hasNext()) {
			boolean more = false;// 此材质是否有多张订单，true代表有多张订单 false代表只是单张订单(包含关联订单)
			boolean freeHour = false;// 是否还有剩余工时
			if (eIndex >= Equipment.length) {
				break;
			}
			if (StandardCapacity <= 0) {// 说明产能已经全部用完
				break;
			}
			List<OrderProcess> moreList = new ArrayList<OrderProcess>();// 当多种任务凑铁水时使用，纪录是哪些工序
			Row row = rows.next();
			String currentID = row.getString("fID");

			String code = row.getString("fStockCode");//物料编码
			String CommonModeSign = row.getString("fCommonModeSign");//共模标识
			String TaskBookID = row.getString("fTaskBookID");//生产任务书ID
			String MaterialType = row.getString("fMaterialType");//材质类型
			String MouldCode = row.getString("fMouldCode");//模具编号
			float OutputRate = row.getDecimal("fOutputRate").floatValue();//工艺出品率
			
			//判断此条数据是否已经列入今日派工计划
			sql = "select a.fID from MES_DispatchedWorkersList a where a.fMainID = '"+ID+"' and a.fProcedurePlanID = '"+currentID+"'";
			Table temp = SQL.select(Sql.DBMySql(sql), null, DATA_MODEL);
			if (temp.size() > 0) {
				continue;
			}

			// 获取订单上的投产数量，以及模具使用性的数量，计算出应该用多少模具，是否会超最大值,计算单个零件的工时是多少
			int putNumber = row.getInt("fNumber");// 投产数量
			int finishNumber = row.getInt("fFinishNumber");// 完工数量
			int rationNum = row.getInt("RationNum");// 模具使用性数量 每个模具产出多少个产品
			if (rationNum == 0) {
				rationNum = 1;
			}
			float TotalHours = row.getDecimal("fTotalHours").floatValue();//总工时
			float singleHour = TotalHours / putNumber;//单件工时
			int Surplus = (int)(StandardCapacity / singleHour / rationNum);//剩余的可生产的数量
			if(Surplus == 0){
				break;
			}

			// 还需要考虑库存数据(待完善)

			int mouldNum = (putNumber - finishNumber) / rationNum;// 模具数量
			int needNum = putNumber - finishNumber;// 需要的数量

			if (needNum > Surplus) {// 如果需要生产的工时小于可生产的工时，生产只能按剩余工时可生产的数量来生产
				freeHour = true;// 说明没有剩余工时，不能再凑其他订单
				needNum = Surplus;
				mouldNum = needNum / rationNum;//模具数量
			}

			if (freeHour) {// 当没有空闲是，直接计算铁水量
				// 直接计算出此订单任务，
			}

			// 根据数量求出铁水的重量
			float weight = tool.calculatedWeight(code, needNum);
			float commonWeigth = LinkedOrder(workerCenterID, currentID, TaskBookID, CommonModeSign, mouldNum, false, null, taskList, null);
			weight += commonWeigth;// 此道工序以及共模订单的总铁水量
			weight = weight / OutputRate;
			int furnaceNum = 0;// 最大开炉数
			int meltingCapacity = 0;// 熔炼量
			Equipment e = null;
			for (int j = 0; j < list.size(); j++) {
				e = list.get(j);
				if (e.getID().equals(Equipment[eIndex])) {
					furnaceNum = e.getMaxTime();// 获取开炉数
					if(furnaceNum == 0 && eIndex < Equipment.length-1){
						eIndex++;
						j=-1;
						continue;
					}
					meltingCapacity = e.getMeltingCapacity();// 设备熔炼量
					break;
				}
			}

			if(furnaceNum == 0){
				break;
			}
			int productionOfMould = 0;// 最终投产的模具数
			if (weight < meltingCapacity && !freeHour) {// 说明不满一炉，继续找相同材质的订单来凑
				sql = "select a.fID,a.fFinishNumber,a.fTotalHours,g.fRationNum as RationNum ,b.fCommonModeSign as fCommonModeSign,b.fTaskBookID as fTaskBookID,b.fStockCode,e.fMaterialType,b.fNumber,sum( f.fNumber ) AS pNumber "
						+ "from MES_ProcedurePlan a "
						+ "left join BAS_WorkCenter h on h.fID = a.fWorkCenterID "
						+ "left join MES_ProductionOrder b on b.fID = a.fMainID "
						+ "left join BAS_StockArchives e on e.fStockCode = b.fStockCode "
						+ "left join  MES_DispatchedWorkersList f ON f.fProcedurePlanID = a.fID "
						+ "left join (SELECT	c.fRationNum,c.fStockCode  FROM	CAPP_MouldProductList c	LEFT JOIN CAPP_Mould d ON d.fID = c.fMainID WHERE d.fStatus = '有效') g ON  g.fStockCode = b.fStockCode "
						+ " where a.fID != '"
						+ currentID
						+ "' and h.fColdOrHot = '是' and a.fIsContinuity = '是'   and b.fOrderStatus = '生产' and a.fPlannedStartDate is not null "
						+ " and e.fMaterialType = '"
						+ MaterialType
						+ "' "
						+ " group by a.fID,a.fFinishNumber,b.fCommonModeSign,b.fTaskBookID,b.fStockCode,e.fMaterialType,b.fNumber,g.fRationNum,a.fTotalHours "
						+ "having b.fNumber > ifnull( sum( f.fNumber ), 0 ) " + "order by a.fPlannedStartDate,b.fRatio  ";

				Table otherOrder = SQL.select(Sql.DBMySql(sql), null, DATA_MODEL);
				if (otherOrder.size() > 0) {// 说明有其他相同材质的
					OrderProcess orderProcess = new OrderProcess(currentID, putNumber - finishNumber, mouldNum, rationNum, code, singleHour);
					moreList.add(orderProcess);
					Iterator<Row> orderRows = otherOrder.iterator();
					while (orderRows.hasNext()) {
						Row orderRow = orderRows.next();
						String otherID = orderRow.getString("fID");
						String otherCode = orderRow.getString("fStockCode");// 存货编码
						int otherNum = orderRow.getInt("fNumber");// 订单投产数量
						int otherFishNum = orderRow.getInt("fFinishNumber");// 完工数量
						int otherRationNum = orderRow.getInt("RationNum");// 模具使用性
						float otherTotalHours = orderRow.getDecimal("fTotalHours").floatValue();// 总工时
						float otherSingelhour = otherTotalHours / (otherNum - otherFishNum);//单件工时
						more = true;
						weight += tool.calculatedWeight(otherCode, otherNum - otherFishNum);
						orderProcess = new OrderProcess(otherID, otherNum - otherFishNum, (otherNum - otherFishNum) / otherRationNum, otherRationNum, code, otherSingelhour);
						moreList.add(orderProcess);
						if (weight >= meltingCapacity) {
							break;
						}
					}
				}
				if (more) {
					moreOrderMakeSureNum(moreList, weight, e, furnaceNum, meltingCapacity);
				} else {
					// 调用方法计算出应该开几炉，计算出模具的数量
					productionOfMould = makeSureNum(code, rationNum, weight, furnaceNum, meltingCapacity, e, more, null, singleHour,mouldNum,MouldCode);
				}

			}else{
				//调用方法计算出应该开几炉，计算出模具数量
				productionOfMould = makeSureNum(code,rationNum,weight,furnaceNum,meltingCapacity,e,false,null,singleHour,mouldNum,MouldCode);
			} 

			// 将排产的任务写入排产计划表
			if (more) {
				moreOrderMakeSureNum(moreList, weight, e, furnaceNum, meltingCapacity);
				for (int i = 0; i < moreList.size(); i++) {
					OrderProcess orderProcess = moreList.get(i);
					if (orderProcess.getMakeNum() > 0) {
						sql = "insert into MES_DispatchedWorkersList (fID,version,fMainID,fProcedurePlanID,fNumber,fWeight,fTotalHours,fMeltingCapacity,fMouldNum,fFinshMouldNum) " + "values(replace(uuid(), '-', ''),0,'" + ID
								+ "','" + orderProcess.getID() + "'," + orderProcess.getMakeNum() + "," + "" + orderProcess.getWeight() + "," + orderProcess.getTotalHours() + ",'" + e.getMeltingCapacity() + "',"+orderProcess.getMouldNum()+",0)";
						SQL.executeUpdate(Sql.DBMySql(sql), null, DATA_MODEL);
						Task task = new Task(orderProcess.getID(), orderProcess.getMakeNum(), e.getID(), orderProcess.getWeight(), orderProcess.getTotalHours());
						taskList.add(task);
						StandardCapacity -= orderProcess.getTotalHours();
					} else {
						break;
					}
				}
			} else {
				sql = "insert into MES_DispatchedWorkersList (fID,version,fMainID,fProcedurePlanID,fNumber,fWeight,fTotalHours,fMeltingCapacity,fMouldNum,fFinshMouldNum) " + "values(replace(uuid(), '-', ''),0,'" + ID + "','"
						+ currentID + "'," + (productionOfMould * rationNum) + "," + tool.calculatedWeight(code, productionOfMould * rationNum) + "," + "" + productionOfMould * rationNum * singleHour
						+ ",'" + e.getMeltingCapacity() + "',"+productionOfMould+",0)";
				SQL.executeUpdate(Sql.DBMySql(sql), null, DATA_MODEL);
				Task task = new Task(currentID, productionOfMould * rationNum, e.getID(), tool.calculatedWeight(code, productionOfMould * rationNum), productionOfMould * rationNum * singleHour);
				taskList.add(task);
				if(commonWeigth > 0){//说明有共模订单，一并列入
					LinkedOrder(workerCenterID,currentID,TaskBookID,CommonModeSign,productionOfMould,true,ID,taskList,e);
				}
				StandardCapacity -= productionOfMould * rationNum * singleHour;
			}

		}
	}

	// 查找相关联订单
	public static float LinkedOrder(String workerCenterID, String currentOrderID, String taskBookID, String sign, int mouldNum, boolean isStart, String ID, List<Task> taskList, Equipment device) {
		float weight = 0;
		String sql = "select a.fID,ifnull((select c.fRationNum from CAPP_MouldProductList c left join CAPP_Mould d on d.fID = c.fMainID where c.fStockCode = b.fStockCode and d.fStatus = '有效'),0) as RationNum "
				+ ",a.fTotalHours,b.fStockCode,b.fNumber from MES_ProcedurePlan a "
				+ "left join MES_ProductionOrder b on b.fID = a.fMainID "
				+ " where a.fID != '"+ currentOrderID
				+ "' and a.fWorkCenterID = '" + workerCenterID + "' and a.fIsContinuity = '是' and b.fCommonModeSign = '" + sign + "' " + " and b.fOrderStatus = '计划'  " + 																																																																														// '否'
				"and b.fTaskBookID = '" + taskBookID + "'";
		Table relevantOrder = SQL.select(Sql.DBMySql(sql), null, DATA_MODEL);
		if (relevantOrder.size() > 0) {
			Iterator<Row> relevantOrderRows = relevantOrder.iterator();
			while (relevantOrderRows.hasNext()) {
				Row relevantOrderRow = relevantOrderRows.next();
				String currentID = relevantOrderRow.getString("fID");
				String code = relevantOrderRow.getString("fStockCode");
				int rationNum = relevantOrderRow.getInt("RationNum");// 模具使用性数量
																		// 每个模具产出多少个产品
				float totalhours = relevantOrderRow.getDecimal("fTotalHours").floatValue();// 总工时
				int orderNum = relevantOrderRow.getInt("fNumber");// 投产数量
				float singelHour = totalhours / orderNum;
				if (rationNum == 0) {
					rationNum = 1;
				}
				weight += tool.calculatedWeight(code, rationNum * mouldNum);
				if (isStart) {
					sql = "insert into MES_DispatchedWorkersList (fID,version,fMainID,fProcedurePlanID,fNumber,fWeight,fTotalHours,fMeltingCapacity,fMouldNum,fFinshMouldNum) " + "values(replace(uuid(), '-', ''),0,'" + ID + "','"
							+ currentID + "'," + (mouldNum * rationNum) + "," + weight + "," + singelHour * mouldNum * rationNum + ",'" + device.getMeltingCapacity() + "',"+mouldNum+",0)";
					SQL.executeUpdate(Sql.DBMySql(sql), null, DATA_MODEL);
					Task task = new Task(currentID, mouldNum * rationNum, device.getID(), weight, singelHour * mouldNum * rationNum);
					taskList.add(task);
					StandardCapacity -= singelHour * mouldNum * rationNum;
				}
			}
		}
		return weight;
	}

	// 多张订单凑铁水
	public static void moreOrderMakeSureNum(List<OrderProcess> moreList, float weight, Equipment e, int furnaceNum, int meltingCapacity) {
		float time = weight / meltingCapacity;// 总铁水量除上熔炼量，算出开炉数
		int intTime = (int) (weight / meltingCapacity);// 取整数
		float decimalTime = time - intTime;// 取小数
		if (intTime == 0) {// 说明不满一炉,由人为处理
			for (int i = 0; i < moreList.size(); i++) {
				OrderProcess orderProcess = moreList.get(i);
				String code = orderProcess.getCode();
				orderProcess.setMakeNum(orderProcess.getNum());
				orderProcess.setWeight(tool.calculatedWeight(code, orderProcess.getNum()));
			}
			e.subtract(1);
			eIndex++;
		} else if (time > furnaceNum) {// 超出了一炉，去掉,//如果开炉数大于了可以开炉的数量，则按可以开炉的次数
			for (int i = 0; i < moreList.size(); i++) {
				OrderProcess orderProcess = moreList.get(i);
				String code = orderProcess.getCode();// 存货编码
				int needNum = orderProcess.getNum();// 需要生产的数量
				float thisWeight = tool.calculatedWeight(code, orderProcess.getNum());// 铁水的重量
				int singleOfMouldNum = orderProcess.getSingleOfMouldNum();
				if (i == 0) {// 第一条数据，默认全部加工
					orderProcess.setMakeNum(orderProcess.getNum());
					orderProcess.setWeight(thisWeight);
					meltingCapacity = (int) (meltingCapacity - thisWeight);
				} else {
					int pNum = (int) (meltingCapacity / thisWeight);// 可以生产的数量
					int inFactNum = pNum > needNum ? needNum : pNum;
					int mould = (int) (inFactNum / singleOfMouldNum);// 模具数量
					orderProcess.setMakeNum(inFactNum);
					orderProcess.setWeight(tool.calculatedWeight(code, inFactNum));
					if (inFactNum == pNum) {
						meltingCapacity = 0;
					}
				}
				if (meltingCapacity == 0) {// 如果全部用完，则
					furnaceNum--;
					e.subtract(1);
					meltingCapacity = e.getMeltingCapacity();
				}
				if (e.getMaxTime() == 0) {
					eIndex++;
					break;
				}
			}
		} else {// 超过一炉，但没有超过最大开炉数
			for (int i = 0; i < moreList.size(); i++) {
				OrderProcess orderProcess = moreList.get(i);
				String code = orderProcess.getCode();// 存货编码
				int needNum = orderProcess.getNum();// 需要生产的数量
				float thisWeight = tool.calculatedWeight(code, orderProcess.getNum());// 铁水的重量
				int singleOfMouldNum = orderProcess.getSingleOfMouldNum();
				if (i == 0) {// 第一条数据，默认全部加工
					orderProcess.setMakeNum(orderProcess.getNum());
					orderProcess.setWeight(thisWeight);
					meltingCapacity = (int) (meltingCapacity - thisWeight);
				} else {
					int pNum = (int) (meltingCapacity / thisWeight);// 可以生产的数量
					int inFactNum = pNum > needNum ? needNum : pNum;
					int mould = (int) (inFactNum / singleOfMouldNum);// 模具数量
					orderProcess.setMakeNum(inFactNum);
					orderProcess.setWeight(tool.calculatedWeight(code, inFactNum));
					if (inFactNum == pNum) {
						meltingCapacity = 0;
					}
				}
				if (meltingCapacity == 0) {// 如果全部用完，则核销开炉数
					furnaceNum--;
					e.subtract(1);
					meltingCapacity = e.getMeltingCapacity();
					if (e.getMaxTime() == 0) {
						eIndex++;
					}
					break;
				}
			}
		}
	}

	
	


	// 当为单个订单使时，确定加工数量
	public static int makeSureNum(String code, int rationNum, float weight, int furnaceNum, int meltingCapacity, Equipment e, boolean more, String type, float singleHour,int maxMouldNum,String MouldCode) {
		int mouldNum = 0;
		// 1.计算出当前铁水需要几炉
		float time = weight / meltingCapacity;// 总铁水量除上熔炼量，算出开炉数
		int intTime = (int) (weight / meltingCapacity);// 取整数
		float decimalTime = time - intTime;// 取小数
		int fireNum = e.getMaxTime();
		float singleWeight = tool.calculatedWeight(code, 1);// 编码的单件重量
		BigDecimal b = new BigDecimal(singleWeight * rationNum); // 每个模具的重量
		float mouldWeght = tool.calculatedMouldWeight(MouldCode);
		// 根据不同情况进行处理
		if (intTime == 0) {// 如果只有一张订单，且不满一炉，自动补满一炉
			if (fireNum >= 1) {
				// 计算出按一炉开需要多少模具
				b = new BigDecimal(singleWeight * rationNum);
				mouldNum = (int) (weight / mouldWeght);
				e.subtract(1);// 核减炉次
				if (e.getMaxTime() == 0) {
					eIndex++;
				}
			}
		}else if(decimalTime < e.getMinMeltingCapacity() && intTime < e.getMaxTime()){//只按一炉开
			if(fireNum >= 1){
				//计算出按一炉开需要多少模具
				b = new BigDecimal(singleWeight * rationNum );
				mouldNum = (int)(meltingCapacity / mouldWeght);
				e.subtract(fireNum);
				if (e.getMaxTime() <= 0) {
					eIndex++;
				}
			}
		} else if(decimalTime > e.getMinMeltingCapacity()  && intTime < e.getMaxTime()){//超出一炉部分，需要满足0.8才能开炉
			if (fireNum > intTime + 1) {
				mouldNum = (int) (meltingCapacity / singleWeight / rationNum) * (intTime + 1);
				if (e.getMaxTime() <= 0) {
					eIndex++;
				}
			} else {
				e.subtract(fireNum);// 核减炉次
				if (e.getMaxTime() <= 0) {
					eIndex++;
				}
				mouldNum = (int) (meltingCapacity / singleWeight / rationNum) * fireNum;
			}
		}else if( intTime >= e.getMaxTime()){//如果任务超过最大开炉数，则按最大开炉数处理
			mouldNum = (int)(e.getMaxTime() * e.getMeltingCapacity() / mouldWeght);
			eIndex++;
			e.setMaxTime(0);
		}
		return mouldNum > maxMouldNum ? maxMouldNum : mouldNum;
	}
	
	public static void selectEquipment(){
		
	}

	// 铁水优先 暂不用
	public static void HotMetalPriority(String workerCenterID) {
		/*
		 * 1.计算出此工作中心优先设定和备选设定的电炉熔炼量有多少 2.将在此工作中心安排的任务按优先级排序
		 * 3.按优先级，逐个对订单进行分析运算，此订单数量需要多少铁水，是否够一炉，不够需要看是否有关联的订单，是否有其他相同材质的订单
		 */

		// 查询与之相关的设备优先级
		List<Equipment> list = new ArrayList<Equipment>();
		int Appoint = 0;// 指定设备的熔炼量
		int Spare = 0;// 备用设备的熔炼量
		int maxAppoint = 0;
		int maxSpare = 0;

		// String sql =
		// "select a.* from MES_ElectricFurnace a where a.fWorkCenterID = '"+workerCenterID+"' order by a.fPriorityRight asc ";
		// Table ElectricFurnace = SQL.select(Sql.DBMySql(sql), null,
		// DATA_MODEL);
		// if(ElectricFurnace.size() > 0){
		// Iterator<Row> ElectricFurnaceRows = ElectricFurnace.iterator();
		// while(ElectricFurnaceRows.hasNext()){
		// Row row = ElectricFurnaceRows.next();
		// Equipment equipment = new Equipment(row.getString("fID"),null, null,
		// 1800,row.getString("fPriorityRight"));
		// list.add(equipment);
		// if("优先设定".equals(row.getString("fPriorityRight"))){
		//
		// }else if("备选设定".equals(row.getString("fPriorityRight"))){
		//
		// }
		// }
		// }
		Appoint = 3000;
		Spare = 1500;
		maxAppoint = 4 * Appoint;
		maxSpare = 6 * Spare;

		String ID = UUID.randomUUID().toString().replace("-", "");// 生成32位字符串
		// 生成派工单
		String ksql = "insert into MES_DispatchedWorkers a (a,a.version,a.fDocumentNumber,a.fStatus,a.fCase,a.fWorkCenterID,a.fDate) " + "values('" + ID
				+ "',0,:nextSeqString(format('%s%s','PG',format('%1$tY%1$tm%1$td', currentDateTime())),'0000'),'未确认','计划优先','" + workerCenterID + "',:currentDate())";
		KSQL.executeUpdate(ksql, null, DATA_MODEL, null);

		// 查出所有与之相关的任务
		String sql = "select a.*,b.fCommonModeSign,b.fTaskBookID from MES_ProcedurePlan a left join MES_ProductionOrder b on a.fMainID = b.fID  " + "where a.fWorkCenterID = '" + workerCenterID
				+ "'  and b.fDocumentNumber like 'SC181127%' order by b.fRatio asc ";// and
																						// a.fIsFinish
																						// =
																						// '否'
		Table table = SQL.select(Sql.DBMySql(sql), null, DATA_MODEL);
		Iterator<Row> rows = table.iterator();
		while (rows.hasNext()) {
			Row row = rows.next();
			String currentID = row.getString("fID");
			String orderID = row.getString("fMainID");
			String commonModeSign = row.getString("fCommonModeSign");// 共模标识
			String taskBookID = row.getString("fTaskBookID");// 生产任务书ID
			sql = "select a.* from MES_DispatchedWorkersList a where a.fMainID = '" + ID + "' and a.fProcedurePlanID = '" + currentID + "' ";
			Table temp = SQL.select(Sql.DBMySql(sql), null, DATA_MODEL);
			if (temp.size() > 0) {
				continue;
			}
			sql = "select sum(a.fPlanNumber) as fPlanNumber from MES_StockPlan a left join MES_ProductionOrder b on a.fMainID = b.fID  " + "where b.fTaskBookID = '" + taskBookID
					+ "'  and b.fCommonModeSign = '" + commonModeSign + "' " + " and b.fOrderStatus = '生产' and a.fMaterialSource = '配方' ";
			System.out.println(sql);
			Table stockPlan = SQL.select(Sql.DBMySql(sql), null, DATA_MODEL);
			if (stockPlan.size() > 0) {
				BigDecimal plan = stockPlan.iterator().next().getDecimal("fPlanNumber");// 计划数量
				System.out.println(plan);
				int fireNum = 0;// 开炉数
				BigDecimal count = plan.divide(new BigDecimal(Appoint));
				fireNum = count.setScale(0, BigDecimal.ROUND_UP).intValue();
				int inToNum = fireNum / (plan.intValue());// 需要生产的数量
				maxAppoint -= Appoint * fireNum;
				// 列入今日派工数据
				sql = "insert into MES_DispatchedWorkersList (fID,version,fMainID,fProcedurePlanID,fNumber) " + "values(replace(uuid(), '-', ''),0,'" + ID + "','" + currentID + "'," + inToNum + ")";
				SQL.executeUpdate(Sql.DBMySql(sql), null, DATA_MODEL);
			}
			if (maxAppoint == 0) {
				break;
			}
		}
	}

	public static void arrange(String ResourceID, Date date, int MeltingAmount, int MaxMeltingAmount, String mainID, String workCenterID) {
		try {
			// 将没有完成的订单找出来
			int initialValue = 0;// 设备所耗初值初值
			String sql = "select a.*,b.fStockCode,b.fCommonModeSign,b.fTaskBookID from MES_ProcedurePlan a left join MES_ProductionOrder b on b.fid = a.fMainID " + "where a.fWorkCenterID = '"
					+ workCenterID + "' and a.fIsFinish = '否'   " + "order by b.fRatio,a.fPlannedStartDate asc";
			Table table = SQL.select(Sql.DBMySql(sql), null, DATA_MODEL);
			Iterator<Row> rows = table.iterator();
			while (rows.hasNext()) {
				// MaxMeltingAmount = MaxMeltingAmount - initialValue;//计算出剩余熔炼量
				Row row = rows.next();
				String currentID = row.getString("fID");// 当前工序ID
				sql = "select a.fid from MES_DispatchedWorkersList a where a.fProcedurePlanID = '" + currentID + "' and a.fMainID = '" + mainID + "'";
				Table procedureTable = SQL.select(Sql.DBMySql(sql), null, DATA_MODEL);
				if (procedureTable.size() > 0) {// 如果大于0，说明已经派工了，无需再次派工
					continue;
				}
				int into = row.getInt("fInToNumber");// 转入数量
				int outto = row.getInt("fFinishNumber");// 转出数量
				int surplus = into - outto;// 剩余需要开工的数量
				String CommonModeSign = row.getString("fCommonModeSign");// 共模标识
				String TaskBookID = row.getString("fTaskBookID");// 生产任务书ID
				String orderID = row.getString("fMainID");// 生产订单ID
				sql = "select a.* from MES_ProcedurePlan a left join MES_ProductionOrder b on b.fid = a.fMainID " + "where a.fWorkCenterID = '" + workCenterID + "' and a.fIsFinish = '否'  "
						+ "and b.fCommonModeSign = '" + CommonModeSign + "' and b.fTaskBookID = '" + TaskBookID + "'";
				// 计算铁水的重量
				sql = "select a.fPlanNumber from MES_StockPlan a  where a.fMainID = '" + orderID + "' and a.fMaterialSource = '配方'  ";
				Table weigthTable = SQL.select(Sql.DBMySql(sql), null, DATA_MODEL);
				BigDecimal weigth = new BigDecimal(0);// 配方量
				if (weigthTable.size() > 0) {
					weigth = weigthTable.iterator().next().getDecimal("fPlanNumber");// 通过配方得到每个产品的单重
					// 此条工序任务的所需铁水量
					BigDecimal need = weigth.multiply(new BigDecimal(surplus));
					if (need.compareTo(new BigDecimal(MaxMeltingAmount)) <= 0) {// 如果熔炼量满足，则安排此工序
						sql = "insert into MES_DispatchedWorkersList (fID,version,fMainID,fProcedurePlanID,fNumber,fResourceID)" + " values(replace(uuid(), '-', ''),0,'" + mainID + "','" + currentID
								+ "'," + surplus + ",'" + ResourceID + "')";
						SQL.executeUpdate(Sql.DBMySql(sql), null, DATA_MODEL);
						MaxMeltingAmount = new BigDecimal(MaxMeltingAmount).divide(weigth).intValue();
					}
				}

			}
		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException(e.getMessage());
		}
	}

	public static String stageDischarge() {
		// ---删除派工数据，正式试用需要删除下列四行代码-------------
		String xxx = "delete from MES_DispatchedWorkersList a";
		KSQL.executeUpdate(xxx, null, DATA_MODEL, null);
		xxx = "delete from MES_DispatchedWorkers a";
		KSQL.executeUpdate(xxx, null, DATA_MODEL, null);
		// ---------------------------------------------------------

		String sql = "select ifnull(a.fOrderProspectPeriod,0) as fOrderProspectPeriod,ifnull(a.fDispatchProspectPeriod,0) as fDispatchProspectPeriod  from MES_Deploy a ";
		Table DeployTable = SQL.select(Sql.DBMySql(sql), null, DATA_MODEL);
		if (DeployTable.size() == 0) {
			// 如果没有找到配置表记录，说明用户没有配置，提示进行配置
			return "norecord";
		}
		int OrderProspectPeriod = DeployTable.iterator().next().getInt("fOrderProspectPeriod");
		if (OrderProspectPeriod == 0) {

		}
		int DispatchProspectPeriod = DeployTable.iterator().next().getInt("fDispatchProspectPeriod");
		if (DispatchProspectPeriod == 0) {

		}
		Calendar calendar = Calendar.getInstance();
		for (int i = 1; i <= DispatchProspectPeriod; i++) {
			discharge(calendar.getTime());
			calendar.add(calendar.DAY_OF_YEAR, 1);
		}
		return "ok";
	}

	/**
	 * 
	 * @Title: queryWorkDispatched
	 * @Description: TODO(查询统计熔炼数据)
	 * @@param fname1设备编号
	 * @@param mater1材质
	 * @@param totalweight1总重量
	 * @@param pmid 时间id
	 * @author DELL
	 * @date 2018-12-21
	 * @return void 返回类型
	 * 
	 */
	public static com.justep.system.data.Table queryWorkDispatched(String fname1, String mater1, String totalweight1, String pmid) {
		String ksql = "select MES_DispatchedWorkersList.*,MES_ProcedurePlan.fCode as Code,MES_ProcedurePlan.fName as Name,MES_ProcedurePlan.fInToNumber as InToNumber,MES_ProcedurePlan.fFinishNumber as FinishNumber,MES_ProcedurePlan.fOutToNumber as OutToNumber,MES_ProcedurePlan.fTotalHours as TotalHours,MES_ProcedurePlan.fPlannedStartDate as PlannedStartDate,MES_ProcedurePlan.fPlannedEndDate as PlannedEndDate,AS_Ledger.fCode as equipmentCode,AS_Ledger.fName as equipmentName,AS_Ledger.fNormsType as NormsType,MES_DispatchedWorkers.fWorkCenterID as WorkCenterID,MES_ProductionOrder.fDocumentNumber as DocumentNumber,MES_ProductionOrder.fStockCode as StockCode,MES_ProductionOrder.fBatch as Batch,BAS_StockArchives.fCustomerCode as CustomerCode,BAS_StockArchives.fStockName as StockName,BAS_StockArchives.fNormsType as codeNormsType,BAS_StockArchives.fMaterialType as MaterialType,BAS_WorkCenter.fName as workerCenterName"
				+ " from MES_DispatchedWorkersList MES_DispatchedWorkersList"
				+ " join MES_DispatchedWorkers MES_DispatchedWorkers on MES_DispatchedWorkersList.fMainID = MES_DispatchedWorkers"
				+ " join MES_ProcedurePlan MES_ProcedurePlan on MES_DispatchedWorkersList.fProcedurePlanID = MES_ProcedurePlan"
				+ " join AS_Ledger AS_Ledger on MES_DispatchedWorkersList.fResourceID = AS_Ledger"
				+ " join MES_ProductionOrder MES_ProductionOrder on MES_ProcedurePlan.fMainID = MES_ProductionOrder"
				+ " join BAS_StockArchives BAS_StockArchives on MES_ProductionOrder.fStockCode = BAS_StockArchives.fStockCode"
				+ " join BAS_WorkCenter BAS_WorkCenter on MES_ProcedurePlan.fWorkCenterID = BAS_WorkCenter" + " where MES_DispatchedWorkersList.fMainID='" + pmid + "' order by AS_Ledger.fNormsType";

		com.justep.system.data.Table table = KSQL.select(ksql, null, DATA_MODEL, null);
		com.justep.system.data.Table tables = TableUtils.createTable("MES_DispatchedWorkersList", "/DB/table/data");
		Iterator<Row> rows = table.iterator();
		tables.addColumns("ID", "String", "name", "String", "times", "String", "mater", "String", "weight", "String", "work", "String", "rate", "String", "disid", "String");
		//System.out.println(table.size());
		Float weight = null;// 重量
		Integer j = 0;// 炉次
		Integer a = 0;// 当前循环数量
		String fwork = "";// 工作中心
		String fcode = "";// 设备编码
		String fcode1 = "";// 设备编码
		String fdis = "";// 子表id
		String c = "";// 材质
		String fnt="";//重量
		Integer totalweight = 0;// 总重量
		DecimalFormat decimalFormat = new DecimalFormat(".000");// 重量
		DecimalFormat decimalFormat1 = new DecimalFormat("0");// 比率
		while (rows.hasNext()) {
			a = a + 1;
			Row row2 = rows.next();
			if (fcode == "") {// 第一次
				fcode = row2.getString("equipmentCode");// 设备编码
				fnt = row2.getString("NormsType");// 设备重量
				fwork = row2.getString("workerCenterName");// 工作中心
				mater1 = row2.getString("MaterialType");// 材质
				fdis = row2.getString("MES_DispatchedWorkersList");// id
				fnt=fnt.subSequence(0, fnt.lastIndexOf("t")).toString();
				int ft=(int) (Double.parseDouble(fnt)*1000);
				if (!totalweight.equals(ft)) {
					j = 0;
					System.out.println(j);
				}
				//System.out.println(fdis);
				// String
				// s=row2.getString("NormsType").substring(0,row2.getString("NormsType").lastIndexOf("t"));
				totalweight = (int) (Double.parseDouble(row2.getString("NormsType").substring(0, row2.getString("NormsType").lastIndexOf("t"))) * 1000);// 总重量
				weight = Float.parseFloat(row2.getDecimal("fWeight").toString());// 获取重量
				// System.out.println(totalweight);
				
				for (int i = 0; i < 10; i++) {// 循环判断是否大于总row2.getString("equipmentName") + "-" + row2.getString("equipmentCode") + "-" + 
					if (weight >= totalweight) {
						weight = weight - totalweight;
						j = j + 1;
						Row row = tables.appendRow();
						row.setString("ID", UUID.randomUUID().toString());
						row.setString("name", row2.getString("NormsType"));
						row.setString("times", j.toString());
						row.setString("mater", mater1);
						row.setString("weight", totalweight.toString());
						row.setString("work", fwork);
						row.setString("rate", "100%");
						if (i == 1) {
							row.setString("disid", row2.getString("MES_DispatchedWorkersList"));
						} else {
							row.setString("disid", fdis);
						}
						//fdis = "";
					} else {
						break;
					}
				}
				String ksql1 = "select MES_DispatchedWorkersList.*,MES_ProcedurePlan.fCode as Code,MES_ProcedurePlan.fName as Name,MES_ProcedurePlan.fInToNumber as InToNumber,MES_ProcedurePlan.fFinishNumber as FinishNumber,MES_ProcedurePlan.fOutToNumber as OutToNumber,MES_ProcedurePlan.fTotalHours as TotalHours,MES_ProcedurePlan.fPlannedStartDate as PlannedStartDate,MES_ProcedurePlan.fPlannedEndDate as PlannedEndDate,AS_Ledger.fCode as equipmentCode,AS_Ledger.fName as equipmentName,AS_Ledger.fNormsType as NormsType,MES_DispatchedWorkers.fWorkCenterID as WorkCenterID,MES_ProductionOrder.fDocumentNumber as DocumentNumber,MES_ProductionOrder.fStockCode as StockCode,MES_ProductionOrder.fBatch as Batch,BAS_StockArchives.fCustomerCode as CustomerCode,BAS_StockArchives.fStockName as StockName,BAS_StockArchives.fNormsType as codeNormsType,BAS_StockArchives.fMaterialType as MaterialType,BAS_WorkCenter.fName as workerCenterName"
						+ " from MES_DispatchedWorkersList MES_DispatchedWorkersList"
						+ " join MES_DispatchedWorkers MES_DispatchedWorkers on MES_DispatchedWorkersList.fMainID = MES_DispatchedWorkers"
						+ " join MES_ProcedurePlan MES_ProcedurePlan on MES_DispatchedWorkersList.fProcedurePlanID = MES_ProcedurePlan"
						+ " join AS_Ledger AS_Ledger on MES_DispatchedWorkersList.fResourceID = AS_Ledger"
						+ " join MES_ProductionOrder MES_ProductionOrder on MES_ProcedurePlan.fMainID = MES_ProductionOrder"
						+ " join BAS_StockArchives BAS_StockArchives on MES_ProductionOrder.fStockCode = BAS_StockArchives.fStockCode"
						+ " join BAS_WorkCenter BAS_WorkCenter on MES_ProcedurePlan.fWorkCenterID = BAS_WorkCenter"
						+ " where MES_DispatchedWorkersList.fMainID='"
						+ pmid
						+ "' and AS_Ledger.fCode='"
						+ fcode + "' and BAS_StockArchives.fMaterialType='" + mater1 + "' order by AS_Ledger.fCode";

				com.justep.system.data.Table table1 = KSQL.select(ksql1, null, DATA_MODEL, null);// 查询次数row2.getString("equipmentName") + "-" + row2.getString("equipmentCode") + "-" + 
				if (a == table1.size()) {
					String frate = decimalFormat1.format((weight / totalweight) * 100) + "%";
					String wei = decimalFormat.format(weight);
					j = j + 1;
					Row row1 = tables.appendRow();
					row1.setString("ID", UUID.randomUUID().toString());
					row1.setString("name", row2.getString("NormsType"));
					row1.setString("times", j.toString());
					row1.setString("mater", mater1);
					row1.setString("weight", wei);
					row1.setString("work", fwork);
					row1.setString("rate", frate);
					row1.setString("disid", fdis);
					a = 0;
					fcode = "";
					fdis = "";
					fcode1 = row2.getString("equipmentCode");// 设备编码
				}
			} else {// 第二次
				if (row2.getString("equipmentCode").equals(fcode)) {// 编码
					if (row2.getString("MaterialType").equals(mater1)) {// 材质
						String fw[] = fwork.split(",");
						fdis = fdis + "," + row2.getString("MES_DispatchedWorkersList");
						if (row2.getString("workerCenterName").equals(fwork)) {// 中心相同
						} else {
							Boolean flag = Arrays.asList(fw).contains(row2.getString("workerCenterName"));
							if (flag) {
							} else {
								fwork = fwork + "," + row2.getString("workerCenterName");
							}
						}
						if(c.equals("2")){
							
						}else{
							c = "1";
						}
						
						weight = weight + Float.parseFloat(row2.getDecimal("fWeight").toString());// 相加重量
						for (int i = 0; i < 10; i++) {// 循环判断是否大于总row2.getString("equipmentName") + "-" + row2.getString("equipmentCode") + "-" + 
							if (weight >= totalweight) {
								weight = weight - totalweight;
								j = j + 1;
								Row row = tables.appendRow();
								row.setString("ID", UUID.randomUUID().toString());
								row.setString("name", row2.getString("NormsType"));
								row.setString("times", j.toString());
								row.setString("mater", mater1);
								row.setString("weight", totalweight.toString());
								row.setString("work", fwork);
								row.setString("rate", "100%");
								if (i > 0) {
									row.setString("disid", row2.getString("MES_DispatchedWorkersList"));
								} else {
									row.setString("disid", fdis);
								}
								c = "2";
								//fdis = "";
							} else {
								break;
							}
						}
						String ksql1 = "select MES_DispatchedWorkersList.*,MES_ProcedurePlan.fCode as Code,MES_ProcedurePlan.fName as Name,MES_ProcedurePlan.fInToNumber as InToNumber,MES_ProcedurePlan.fFinishNumber as FinishNumber,MES_ProcedurePlan.fOutToNumber as OutToNumber,MES_ProcedurePlan.fTotalHours as TotalHours,MES_ProcedurePlan.fPlannedStartDate as PlannedStartDate,MES_ProcedurePlan.fPlannedEndDate as PlannedEndDate,AS_Ledger.fCode as equipmentCode,AS_Ledger.fName as equipmentName,AS_Ledger.fNormsType as NormsType,MES_DispatchedWorkers.fWorkCenterID as WorkCenterID,MES_ProductionOrder.fDocumentNumber as DocumentNumber,MES_ProductionOrder.fStockCode as StockCode,MES_ProductionOrder.fBatch as Batch,BAS_StockArchives.fCustomerCode as CustomerCode,BAS_StockArchives.fStockName as StockName,BAS_StockArchives.fNormsType as codeNormsType,BAS_StockArchives.fMaterialType as MaterialType,BAS_WorkCenter.fName as workerCenterName"
								+ " from MES_DispatchedWorkersList MES_DispatchedWorkersList"
								+ " join MES_DispatchedWorkers MES_DispatchedWorkers on MES_DispatchedWorkersList.fMainID = MES_DispatchedWorkers"
								+ " join MES_ProcedurePlan MES_ProcedurePlan on MES_DispatchedWorkersList.fProcedurePlanID = MES_ProcedurePlan"
								+ " join AS_Ledger AS_Ledger on MES_DispatchedWorkersList.fResourceID = AS_Ledger"
								+ " join MES_ProductionOrder MES_ProductionOrder on MES_ProcedurePlan.fMainID = MES_ProductionOrder"
								+ " join BAS_StockArchives BAS_StockArchives on MES_ProductionOrder.fStockCode = BAS_StockArchives.fStockCode"
								+ " join BAS_WorkCenter BAS_WorkCenter on MES_ProcedurePlan.fWorkCenterID = BAS_WorkCenter"
								+ " where MES_DispatchedWorkersList.fMainID='"
								+ pmid
								+ "' and AS_Ledger.fCode='" + fcode + "' and BAS_StockArchives.fMaterialType='" + mater1 + "' order by AS_Ledger.fCode";

						com.justep.system.data.Table table1 = KSQL.select(ksql1, null, DATA_MODEL, null);// 查询次数row2.getString("equipmentName") + "-" + row2.getString("equipmentCode") + "-" + 
						if (a == table1.size()) {
							String frate = decimalFormat1.format((weight / totalweight) * 100) + "%";
							String wei = decimalFormat.format(weight);
							j = j + 1;
							Row row1 = tables.appendRow();
							row1.setString("ID", UUID.randomUUID().toString());
							row1.setString("name", row2.getString("NormsType"));
							row1.setString("times", j.toString());
							row1.setString("mater", mater1);
							row1.setString("weight", wei);
							row1.setString("work", fwork);
							row1.setString("rate", frate);
							if (c.equals("2")) {
								row1.setString("disid", row2.getString("MES_DispatchedWorkersList"));
							} else {
								row1.setString("disid", fdis);
							}
							a = 0;
							fcode = "";
							fdis = "";
							fcode1 = row2.getString("equipmentCode");// 设备编码
						}
					}
				}
			}
		}
		return tables;
	}

	/**
	 * 
	 * @Title: queryWorkDispatched
	 * @Description: TODO(确认排产状态)
	 * @author lws
	 * @date 2018-12-21
	 * @return void 返回类型
	 * 
	 */
	public static String confirmDiswork(){
			try {
				String sql1="select count(1) as wnum from mes_dispatchedworkers dw where dw.fDate='"+new SimpleDateFormat("yyyy-MM-dd").format(new Date())+"' and dw.fStatus='已确认'";
				Table t=SQL.select(sql1, null, DATA_MODEL);
				
				Integer count=t.iterator().next().getInteger("wnum");
				if(count==4){
					return "fal";//已确认
				}else{//未确认
					String sql="update mes_dispatchedworkers dw set dw.fStatus='已确认' where dw.fWorkCenterID not in (select bw.fID from bas_workcenter bw where bw.fIsMelting='是') and dw.fDate='"+new SimpleDateFormat("yyyy-MM-dd").format(new Date())+"'";
					SQL.executeUpdate(sql, null, DATA_MODEL);
				}	
			} catch (Exception e) {
				
				e.printStackTrace();
				 return e.getMessage();
			}
			return "ok";
	}

	/**
	 * 
	 * @Title: queryWorkDispatched
	 * @Description: TODO(计算并添加排产量)
	 * @@param pid 工序id
	 * @author lws
	 * @date 2018-12-21
	 * @return void 返回类型
	 * 
	 */
	public static String calWDis(String dwid,String perid,Integer dnum,String mcode,String scode,String pid,String wid,String date){
		String ID = UUID.randomUUID().toString().replace("-", "");// 生成32位字符串
		
			try {
				
				String sqld="select md.fID from mes_dispatchedworkers md where md.fWorkCenterID='"+wid+"' and md.fDate = '"+date+"'";//查找排产主表id
				Table tabled=SQL.select(sqld, null, DATA_MODEL);//模具
				if(tabled.iterator().hasNext()){
					Row rowd=tabled.iterator().next();
					dwid=rowd.getString("fID");
				}
			
				if(null==dwid || ("undefined").equals(dwid)){//判断计划排产主表id是否存在
					
					String ksql = "insert into MES_DispatchedWorkers a (a,a.version,a.fDocumentNumber,a.fStatus,a.fCase,a.fWorkCenterID,a.fDate) " + "values('" + ID
							+ "',0,:nextSeqString(format('%s%s','PG',format('%1$tY%1$tm%1$td', currentDateTime())),'0000'),'未确认','计划优先','" + wid + "','"+date + "')";				
					KSQL.executeUpdate(ksql, null, DATA_MODEL, null);
					dwid=ID;
				}
				
				String sqlb="select * from capp_bomlist bl where bl.fMaterialSource='配方' and bl.fMainID =(select cb.fID from capp_bom cb where cb.fStockCode='"+scode+"')";//获取bom明细的比列计算铁水重量
				Table tableb=SQL.select(sqlb, null, DATA_MODEL);//bom
				Row rowb=tableb.iterator().next();
				Double tolb=Double.parseDouble(rowb.getDecimal("fRationNum").toString())*Double.parseDouble(dnum.toString());//获取bom定额用量*订单数量(总铁水重量)
				
				
				String sqlm="select * from capp_mouldproductlist cml where cml.fStockCode='"+scode+"' and cml.fMainID =(select cm.fID from capp_mould cm where cm.fStockCode='"+mcode+"')";//获取模具明细的比例来计算模具数量
				Table tablem=SQL.select(sqlm, null, DATA_MODEL);//模具
				Row rowm=tablem.iterator().next();
				
				Integer tolm=dnum/rowm.getInteger("fRationNum");//订单数量除获取模具定额用量(总模具数量)

				
				String sql="insert into MES_DispatchedWorkersList (fID,version,fMainID,fProcedurePlanID,fNumber,fCreatePersonID,fCreateDateTime,fLastUpdatePersonID,fLastUpdateDateTime,fWeight,fFinishNumber,fMouldNum,fFinshMouldNum)  values('"+UUID.randomUUID().toString().replace("-", "")+"',0,'"+dwid+"','"+pid+"','"+dnum+"','"+perid+"','"+new SimpleDateFormat("yyyy-MM-dd hh:mm:ss").format(new Date())+"','"+perid+"','"+new SimpleDateFormat("yyyy-MM-dd hh:mm:ss").format(new Date())+"','"+tolb+"','0','"+tolm+"','0')";	//新添加排产明细数据
				
				SQL.executeUpdate(sql, null, DATA_MODEL);
				
			} catch (Exception e) {
				
				e.printStackTrace();
				
				return e.getMessage();
			}
		
			return "ok";
	}
	
	
	
	/**
	 * 
	 * @Title: queryWorkDispatched
	 * @Description: TODO(获取熔炼量)
	 * @@param pid 工序id
	 * @author lws
	 * @date 2018-12-21
	 * @return void 返回类型
	 * 
	 */
	public static Table getMeCa(String mc){
			String sql="select al.fMeltingCapacity as fMeCa from as_ledger al group by al.fMeltingCapacity";
			
			Table table=SQL.select(sql, null, DATA_MODEL);
			
			return table;
	}
}