import java.math.*;
import java.sql.*;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.Date;

import org.dom4j.*;
import com.justep.system.data.*;
import com.justep.system.data.Table;
import java.util.Calendar;

public class ProductionPlan {
	private static final String DATA_MODEL = "/DB/table/data";//表的模块
	private static int index, count, most_compressed_count; // 零件序号，零件个数，
	// 最多压缩零件个数
	private static double longest, to_compress, min_rate, temp_rate; // 最长路径加工总时间，
				                                                     // 可压缩时间，
			                                                         // 最小压缩比，
				                                                     // 临时压缩比
	private static boolean all_fixed, unable_to_finish; // 全部不可压缩， 不能完成压缩
	private static String errormessage = "error";// 错误提示
	private static String errorOrderNumber = null;

	public static String plannedScheduling(String orderIDs){
		try {
			String ksql = "select p, p.fRelevantID from MES_TreeOrder p where " + orderIDs + ""; // 根据生产订单ID找出成品件
			Table table = KSQL.select(ksql, null, DATA_MODEL, null);
			if (table.size() > 0) { // 遍历成品件
				Iterator<Row> rows = table.iterator();
				while (rows.hasNext()) {
					Row row = rows.next();

					index = 1;
					count = 0;
					longest = 0;
					most_compressed_count = 0;
					to_compress = 0;
					all_fixed = false;
					unable_to_finish = false;
					int limit, root; // 工期时长，根节点
					int father[] = new int[10005], path[] = new int[10005], most_compressed_path[] = new int[10005]; // 该零件的父级零件序号，
																														// 当前路径，
																														// 可压缩路径
					boolean compressed[] = new boolean[10005]; // 是否为关键路径
					double length[] = new double[10005], min_length[] = new double[10005], original_length[] = new double[10005]; // 订单时长，
																																	// 最小时长，
																																	// 最初时长

					int[][] tree = new int[1005][1005]; // 父子关系二维数组
					String[] produceOrderID = new String[10005]; // 订单ID

					ksql = "select p,p.fSafetyLead, p.fDeliveryDate, p.fDocumentNumber,p.fNumber from MES_ProductionOrder p where p = '"
							+ row.getString("fRelevantID") + "'"; // 找出生产订单的标准提前期
					Table table2 = KSQL.select(ksql, null, DATA_MODEL, null);
					if (table2.size() > 0) {
						Row row2 = table2.iterator().next();
						String ordernumber = row2.getString("fDocumentNumber");//生产订单号
						String orderID = row2.getString("p");//生产订单ID
						errorOrderNumber = ordernumber;
						if (row2.getDate("fDeliveryDate") == null) {
							errormessage = "【" + ordernumber + "】订单没有交货期!";
						}
						// 根据默认日历，计算当前日期至交货期之间的工作日
						String ksql2 = "select p from BAS_CalendarList p where SQL.date(p.fDate) between :dateFormat(:currentDate(), 'yyyy-MM-dd') and '"
								+ row2.getDate("fDeliveryDate").toString() + "'"
								+ "and p.fIsWorkingDay = '是' and p.fMainID = (select q from BAS_Calendar q where q.fIsDefaultCalendar = '是')";
						Table table3 = KSQL.select(ksql2, null, DATA_MODEL, null);
						if (table3.size() > 0) {
							limit = table3.size(); // 当前日期至交货期之间的工作日即为生产成品的最长时间
							root = index; // 根节点即为第一个生产订单
							father[index] = index; // 当前订单的父订单序号根节点的序号即为自己
							errormessage = "【" + ordernumber + "】订单没有标准提前期!";
							//int orderlength = orderTime(row2.getString("p"), ordernumber, row2.getInt("fNumber"));
							int orderlength = orderTimeTest(row2.getString("p"), ordernumber, row2.getInt("fNumber"));
							//orderTimeTest(row2.getString("p"), ordernumber, row2.getInt("fNumber"));
							// // 给订单时长赋值
							length[index] = orderlength + row2.getInt("fSafetyLead");
							original_length[index] = length[index]; // 给最初时长赋值
							errormessage = "【" + ordernumber + "】订单没有设置不可压缩时间!";
							min_length[index] = orderMinTime(orderID,row2.getInt("fNumber")); // 给最小时长赋值
							errormessage = "error";
							produceOrderID[index] = row.getString("fRelevantID"); // 给订单ID数组赋值
							traverseResoucesChain(row.getString("p"), index, father, length, original_length, produceOrderID, tree, min_length); // 遍历资源链路

							path[count = 1] = root; // 当前路径赋值
							min_rate = 1; // 初始化最小压缩比
							dfs(root, length[root], tree, path, length, limit, compressed, min_length, most_compressed_path);

							while (longest > limit) {
								if (unable_to_finish) {
									return "unable_to_finish";
								}
								for (int i = 1, p; i <= most_compressed_count; i++) {
									p = most_compressed_path[i];
									if (!compressed[p]) {
										compressed[p] = true;
										length[p] = (length[p] - min_length[p]) * min_rate + min_length[p];
									}
								}
								min_rate = 1;
								longest = 0;
								dfs(root, length[root], tree, path, length, limit, compressed, min_length, most_compressed_path);
							}
							
							Calendar calendarEnd = Calendar.getInstance();
							calendarEnd.setTime(new SimpleDateFormat("yyyy-MM-dd").parse(row2.getDate("fDeliveryDate").toString()));
							calendarEnd.add(Calendar.DATE, -(int) (length[root] - 1));

							int noWorkingDay = calculateIsNotWorkingDay(new SimpleDateFormat("yyyy-MM-dd").format(calendarEnd.getTime()), row2.getDate("fDeliveryDate").toString());
							calendarEnd.add(Calendar.DATE, -noWorkingDay);

							
							//float orderSt = (float) Math.round(((int) length[root]/orderlength)*100)/100;//生产订单的压缩比
							float orderSt = (float)length[root]/(float)orderlength;//生产订单的压缩比
							BigDecimal b = new BigDecimal(orderSt);  
							orderSt = b.setScale(2,BigDecimal.ROUND_HALF_UP).floatValue(); 
//							if(orderSt < 1){
//								flag = true;
//							}
							if(orderSt == 0){
								orderSt = 0.01f;
							}
							String ksql3 = "";
							if(IsStart(produceOrderID[root])){//如果已有完工工序，则计划开始日期不变
								ksql3 = "update MES_ProductionOrder p set p.fRatio = "	+ length[root]/orderlength	+ ", p.fPlannedEndDate = :toDate('"
										+ calculateWorkingDay(getCalendar(row2.getDate("fDeliveryDate").toString()), "enddate")
										+ "') , p.fStandardCycle = "+orderlength+" "
										+ " ,p.fPlannedCycle = " + (int) length[root] + " where p = '" + produceOrderID[root] + "'";
							}else{
								ksql3 = "update MES_ProductionOrder p set p.fRatio = "	+ length[root]/orderlength
										+ ", p.fPlannedStartDate = :toDate('"
										+ calculateWorkingDay(calendarEnd, "startDate")
										+ "') , p.fPlannedEndDate = :toDate('"
										+ calculateWorkingDay(getCalendar(row2.getDate("fDeliveryDate").toString()), "enddate")
										+ "') , p.fStandardCycle = "+orderlength+" "
										+ " ,p.fPlannedCycle = " + (int) length[root] + " where p = '" + produceOrderID[root] + "'";
							}
							isContinuity(produceOrderID[root]);
							KSQL.executeUpdate(ksql3, null, DATA_MODEL, null);
							//updateProcedurePlan(produceOrderID[root], new SimpleDateFormat("yyyy-MM-dd").format(calendarEnd.getTime()), row2.getDate("fDeliveryDate").toString(),new BigDecimal(length[root]/orderlength), 1);
							updateProcedurePlan(produceOrderID[root], new SimpleDateFormat("yyyy-MM-dd").format(calendarEnd.getTime()), row2.getDate("fDeliveryDate").toString(), orderSt);
							
							calendarEnd.add(Calendar.DATE, -1);
							traverseTree(root, new SimpleDateFormat("yyyy-MM-dd").format(calendarEnd.getTime()), tree, length, produceOrderID, original_length, min_length);
						}
					}

				}
			} else {
				errormessage = "此订单不在资源链路中";
				int x = 1 / 0;
			}
		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException(errormessage);
		}
		return "success";
	}
	
	/**
	 * 判断此订单是否有已完工工序
	 * @param orderID
	 * @return
	 */
	public static boolean IsStart (String orderID){
		String ksql = "select count(a) as c from MES_ProcedurePlan a where a.fMainID = '"+orderID+"' and a.fActualEndDate is not null ";
		Table t = KSQL.select(ksql, null, DATA_MODEL, null);
		int count = t.iterator().next().getInt("c");
		if(count > 0){
			return true;
		}
		return false;
	}
	
	/**
	 * 计算这个订单最新的标准提前期，计划运算时调用
	 * 
	 * @param orderID
	 *            生成订单的ID
	 * @return
	 */
	public static int orderTime(String orderID, String orderNumber, int resultNum) {
		String ksql = "select a.*,b.fClassesNumber,b.fAfterDay,b.fBeforeDay,b.fColdOrHot,b.fTonnage " +
				"from MES_ProcedurePlan a optional join BAS_WorkCenter b on b = a.fWorkCenterID " +
				"where  a.fMainID = '" + orderID + "' and a.fIsFinish = '否'  ";//and a.fFinishNumber < "+resultNum+"
		Table t = KSQL.select(ksql, null, DATA_MODEL, null);
		Iterator<Row> rows = t.iterator();
		int s = 0;
		while (rows.hasNext()) {
			Row row = rows.next();
			Date ActualStartDate = row.getDateTime("fActualStartDate");//实际开始日期
			int FinishNumber = row.getInt("fFinishNumber");//完工数量
			int time = row.getInt("fStandardCycle");// 工序的标准提前期
			int EachGroupNumber = row.getInt("fEachGroupNumber");//每组数量
			float SingleGroupHour = row.getDecimal("fSingleGroupHour").floatValue();//单组工时
			int ClassesNumber = row.getInt("fClassesNumber");//班次数
			int AfterDay = row.getInt("fAfterDay");//工作中心后置等待时间
			int beforeDay = row.getInt("fBeforeDay");//工作中心前置等待时间
			int OutToNumber = row.getInt("fOutToNumber");//工艺周期
			String ColdOrHot = row.getString("fColdOrHot");//热线 / 冷线
			BigDecimal TotalHours = row.getDecimal("fTotalHours");//总工时
			BigDecimal QuasiJunctions = row.getDecimal("fQuasiJunctions");//准结工时
			int ResourcesNumber = row.getInt("fResourcesNumber");//资源数
			
			int workingHours = 0;//工时时间
			//按工时计算标准生产周期
			if(ActualStartDate == null){//如果开始日期为空，则说明还没开始
				int temp = (int)((resultNum /EachGroupNumber)*SingleGroupHour + QuasiJunctions.intValue()) /60/(ClassesNumber*8)/ResourcesNumber;
				workingHours = temp + beforeDay +AfterDay + OutToNumber/24;
				//s = s + time;
			}else{
				//去掉工作中心的前等待时间和准结时间，重新运算此工序的生产周期
				int temp = (int)(((resultNum - FinishNumber)/EachGroupNumber)*SingleGroupHour/60/(ClassesNumber*8)/ResourcesNumber);
				workingHours = temp + beforeDay +AfterDay + OutToNumber/24;
				
			//	s = s + time;
			}
			if(workingHours == 0){
				workingHours = 1;
			}
			
			int smeltingTime = 0;//熔炼时间
			
			//如果是热线，则还需要考虑熔炼量
			ksql = "select a.* from MES_StockPlan a where a.fMainID = '"+orderID+"' ";
			Table StockPlan = KSQL.select(ksql, null, DATA_MODEL, null);
			if("热线".equals(ColdOrHot) && StockPlan.size() > 0 ){
				BigDecimal Tonnage = row.getDecimal("fTonnage");//熔炼量
				BigDecimal Consumption = StockPlan.iterator().next().getDecimal("fPlanNumber");//铁水用量
				Consumption = Consumption.divide(new BigDecimal(1000));
				smeltingTime = Consumption.divide(Tonnage,0,BigDecimal.ROUND_HALF_UP).intValue();
			}
			//System.out.println("工时时间 " + workingHours + "熔炼时间 "+smeltingTime);
			
			time = workingHours > smeltingTime ? workingHours : smeltingTime;
			s += time;
			
			
			ksql = "update MES_ProcedurePlan a set a.fStandardCycle = "+time+" where a = '"+row.getString("a")+"'";
			KSQL.executeUpdate(ksql, null, DATA_MODEL, null);
		}
		return s;
	}
	
	private static HashMap<String, int[][]> map = new HashMap<String, int[][]>();
	/**
	 * 流水方式运算每道工序的标准提前期以及订单的标准提前期
	 * 
	 * @param orderID
	 *            生成订单的ID
	 * @return
	 */
	public static int orderTimeTest(String orderID, String orderNumber, int resultNum) {
//		String ksql = "select a.*, SQL.ifnull(a.fTransferNumber,-1) as TransfersNum ,b.fAfterDay,b.fBeforeDay " +
//				"from MES_ProcedurePlan a optional join BAS_WorkCenter b on b = a.fWorkCenterID " +
//				"where a.fMainID = '" + orderID + "' and a.fIsFinish = '否' order by a.fCode asc ";
//		Table t = KSQL.select(ksql, null, DATA_MODEL, null);
//		Iterator<Row> rows = t.iterator();
//		int s = 0;
//		
//		int i =1;// 数组行下标
//		int j = 1;//数据列下标	
//		int[][] timeArray = new int[t.size()+1][1000];//存放工序的进度表
//		map.put(orderID, timeArray);//将工序流水排的结果存入hashmap中
//		
//		int dayHour = 0;
//		int previous = -1;
//		int previousSign = 0;
//		int previousEndSign = 0;
//		float previousNumber = 0;
// 		while (rows.hasNext()) {
//			Row row = rows.next();
//			int time = row.getInt("fStandardCycle");// 工序的标准提前期
//			String Continuity = row.getString("fIsContinuity");//连续标识
//			int ContinuityFlag = 1;	
//			if("是".equals(Continuity))ContinuityFlag = 0;
//					
//			float TotalManhour = row.getDecimal("fTotalHours").floatValue();//总工时
//			int CompletionQuantity = row.getInt("fFinishNumber");//完工数量
//			//float SplitNumber = row.getFloat("fSplitNumber");//拆分数量
//			int fResourceNumber = row.getInt("fResourcesNumber");//资源数
//			int fTransfersNum = row.getInt("TransfersNum");//中转数量
//			String workerCenterID = row.getString("fWorkCenterID");//工作中心ID
//			Date startDate = row.getDateTime("fActualStartDate");//实际开工日期
//			if(workerCenterID == null){
//				errormessage = "【" + orderNumber + "】订单缺少工作中心，不能运算，请先维护工作中心!";
//			}
//			int ClassesNumber = row.getInt("fClassesNumber");//班次数
//			int AfterWaitingTime = row.getInt("fAfterDay");//后置等待时间
//			int fBeforeWaitingTime = row.getInt("fBeforeDay");//前置等待时间
//			
//			if(CompletionQuantity > 0 ){
//				//说明已经有部分做完，需要去掉这部分零件的加工的时间
//				TotalManhour = TotalManhour - (TotalManhour / resultNum * CompletionQuantity );//剩余工时
//				TotalManhour=TotalManhour/fResourceNumber;//总工时/资源数
//				float addTime = 0;
//				if(ContinuityFlag == 1){//独立工序，加前后时间
//					addTime = fBeforeWaitingTime + AfterWaitingTime;
//				}else if(ContinuityFlag == 4){
//					addTime = AfterWaitingTime;
//				}else{
//					addTime = 0;
//				}
//				
//				time = (int) Math.ceil(TotalManhour / (ClassesNumber*8) + addTime);
//				ksql = "update MES_ProcedurePlan a set a.fStandardCycle = "+time+" where a = '"+row.getString("a")+"'";
//				KSQL.executeUpdate(ksql, null, DATA_MODEL, null);
//			}else{
//				//加前置日期
//				//if(fResourceNumber>1){
//				float addTime = 0;
//				if(ContinuityFlag == 1){//独立工序，加前后时间
//					addTime = AfterWaitingTime + fBeforeWaitingTime;
//				}else if(ContinuityFlag == 2){//连续工序的第一道工序，只加前等
//					addTime = fBeforeWaitingTime;
//				}else if(ContinuityFlag == 4){//连续工序的第二道工序，只加后等
//					addTime = AfterWaitingTime;
//				}else{
//					addTime = 0;
//				}
//				TotalManhour = TotalManhour/fResourceNumber;//总工时/资源数
//				time = (int) Math.ceil(TotalManhour / (ClassesNumber*8) + addTime);
//				ksql = "update MES_ProcedurePlan a set a.fStandardCycle = "+time+" where a = '"+row.getString("a")+"'";
//				KSQL.executeUpdate(ksql, null, DATA_MODEL, null);
//				//}
//			}
//			
//			
//			if(i == 1){//如果是一道工序，从1开始
//				j = 1;
//			}else if(previous == -1){//如果没有流水作业，则直接在下一天开始
//				j++;				
//			}else if(startDate != null){//说明已经开工
//				j = 1;
//			}else{
//				//计算开始时间
//				if(previous >= 0){
//					j = previousSign + previous;
//				}
//			}
//			timeArray[i][j] = 9;//开始时间得标记点
//			float one = TotalManhour / resultNum;
//			int flowTime = (int)((one * fTransfersNum) / (ClassesNumber * 8));//记录当前中转的批量时间
//			if(flowTime == 0) flowTime = 1 ;
//			int thisFlowTime = (int)((one * previousNumber) / (ClassesNumber * 8));//记录当前中转的批量时间
//			if(thisFlowTime == 0) thisFlowTime = 1 ;
//			if(fTransfersNum == -1){
//				flowTime = -1;
//			}
//			previousSign = j;//当前开始的标记
//			
//			//计算结束时间
//			if(previous == -1){//说明不流水作业，直接加上提前期
//				j += time - 1;
//			}else if(flowTime != -1 && previous > flowTime){//比较上下工序哪个流水的批量时间更大
//				j = previousEndSign + flowTime;
//			}else if(flowTime != -1 && previous < flowTime){
//				j += time - 1;
//			}else{
//				j += time - 1;
//				if(j < previousEndSign){
//					j = previousEndSign + thisFlowTime;
//				}
//			}
//			
//			previous = flowTime;
//			previousEndSign = j;
//			timeArray[i][j] = 9;//结束时间得标记点
//			previousNumber = fTransfersNum;
//			i++;
//		}
// 		for(int x = 1;x<timeArray.length;x++){
// 			for(int y = 1;y<timeArray[x].length;y++){
// 				if(timeArray[x][y] == 0){
// 					System.out.print("0");
// 				}else{
// 					System.out.print("_");
// 				}
// 			}
// 			System.out.println();
// 		}
// 		s = j;
		
		
		
		String ksql = "select a.*, SQL.ifnull(a.fTransferNumber,-1) as TransfersNum ,b.fAfterDay,b.fBeforeDay,b.fColdOrHot,b.fTonnage " +
				"from MES_ProcedurePlan a optional join BAS_WorkCenter b on b = a.fWorkCenterID " +
				"where a.fMainID = '" + orderID + "' and a.fIsFinish = '否' order by a.fCode asc ";
		Table t = KSQL.select(ksql, null, DATA_MODEL, null);
		Iterator<Row> rows = t.iterator();
		int s = 0;
		
		int i =1;// 数组行下标
		int j = 1;//数据列下标	
		int[][] timeArray = new int[t.size()+1][1000];//存放工序的进度表
		map.put(orderID, timeArray);//将工序流水排的结果存入hashmap中
		
		float dayHour = 0;//每天按8小时计算的天数
		int previousStart = 0;//上道开始时间
		int previousEnd = 0;//上道结束时间
		float previousHour = 0;//上道的批量工时
		String previousIsContinuity = "";//上道的连续标识
		boolean flowFlag = false;//流水作业标识
		//int startPoint = 0;//每道工序的起始点
		int previousNumber = 0;
		//如果是热线，则还需要考虑熔炼量
		ksql = "select a.* from MES_StockPlan a where a.fMainID = '"+orderID+"' ";
		Table StockPlan = KSQL.select(ksql, null, DATA_MODEL, null);
		
 		while (rows.hasNext()) {
			Row row = rows.next();
			String ColdOrHot = row.getString("fColdOrHot");//热线还是冷线
			String IsContinuity = row.getString("fIsContinuity");//是否连续
			BigDecimal TotalHours = row.getDecimal("fTotalHours");//总工时
			int ClassesNumber = row.getInt("fClassesNumber");//运算班次数
			int FinishNumber = row.getInt("fFinishNumber");//完工数量
			int ResourcesNumber = row.getInt("fResourcesNumber");//资源数
			int ProcessCycle = row.getInt("fProcessCycle");//工艺周期
			int TransfersNum = row.getInt("TransfersNum");//中转数
			int code = row.getInt("fCode");//工序号
			int AfterDay = row.getInt("fAfterDay");//后置等待时间
			int BeforeDay = row.getInt("fBeforeDay");//前置等待时间
			int time = 0;//以天计算当前工序的时间
			int hours = 0;//已小时计算当前工序的时间	
			float singletonHour = TotalHours.divide(new BigDecimal(resultNum),2,BigDecimal.ROUND_HALF_UP).floatValue();//单件工时         总工时 / 总数量
			
			//获取之前工序的报废数量
			ksql = "select SQL.ifnull(sum(a.fScrapNumber),0) as fScrapNumber from MES_ProcedurePlan a where a.fMainID = '"+orderID+"' and a.fCode < "+code+" ";
			Table ScrapTable = KSQL.select(ksql, null, DATA_MODEL, null);
			int ScrapNumber = ScrapTable.iterator().next().getDecimal("fScrapNumber").intValue();//总报废数
			int residualQuantity = resultNum - FinishNumber - ScrapNumber; //剩余数量
			float residual = residualQuantity * singletonHour ;//剩余工时
			
			int smeltingTime = 0;//熔炼的天数		
			if("热线".equals(ColdOrHot) && StockPlan.size() > 0){
				BigDecimal Tonnage = row.getDecimal("fTonnage");//熔炼量
				BigDecimal Consumption = StockPlan.iterator().next().getDecimal("fPlanNumber");//铁水用量
				Consumption = Consumption.divide(new BigDecimal(1000));
				smeltingTime = Consumption.divide(Tonnage,0,BigDecimal.ROUND_HALF_UP).intValue();
			}
			float singletonDay = (residual / ResourcesNumber)  / (ClassesNumber * 8) + ProcessCycle / 24;//工时的天数
			//System.out.println(singletonDay);
			float currentTime = smeltingTime > singletonDay ? smeltingTime : singletonDay;//当前工序的时常（不含前后等待）
			
			int freeTime = 0;
			
			int currentStart = 0;
			int currentEnd = 0;
			//判断要加的等待时间
			if("是".equals(IsContinuity)){
				//如果是连续工序，则需要加上前置等待
				freeTime = BeforeDay + AfterDay;
			}else if("否".equals(IsContinuity) && "是".equals(previousIsContinuity) ){
				//如果上道是连续工序，本道不是连续工序，说明当前工序是连续的最后一道，则需要加上后置时间
				freeTime = AfterDay;
			}else{
				//说明是独立工序，需要加上前置和后置等待时间
				freeTime = AfterDay + BeforeDay;
			}
			if("是".equals(previousIsContinuity)){
				currentStart = previousStart;
				currentEnd = previousEnd;
				timeArray[i][currentStart] = 9;
				timeArray[i][currentEnd] = 9;
			}else if(flowFlag){//说明是流水作业
				//比对前后分批时间，然后加上前置时间
				if(previousHour < 1){
					currentStart = previousStart;
				}else{
					currentStart = previousStart + (int)previousHour;
				}
				timeArray[i][currentStart] = 9;
				float current = (previousNumber * singletonHour)/ (ClassesNumber * 8);
				if(current > previousHour){//如果当前分批时间比之前的大，则，加上当前工序的时间
					if( current < 1){//如果小于1，则一天能完成
						currentEnd = currentStart;
						dayHour = current;
					}else{
						currentEnd = currentStart + (int)current;
						dayHour = current - (int)Math.floor(current);//取小数
					}
				}else{//如果当前比之前的小，则在上道结束的基础上加上当前的分批时间
					if(current < 1){//如果小于1，则一天能完成
						currentEnd = previousEnd;
						dayHour = current;
					}else{
						currentEnd = previousEnd + (int)current;
						dayHour = current - (int)Math.floor(current);//取小数
					}
				}
				timeArray[i][currentEnd] = 9;
			}else{//不是流水作业
				if(dayHour == 0){
					currentStart = previousEnd + 1;
				}else{
					currentStart = previousEnd;
				}
				timeArray[i][currentStart] = 9;
				float decimal = currentTime - (int)Math.floor(currentTime);
				if((decimal + dayHour) > 1 ){//说明要加一天
					currentEnd = currentStart + freeTime + (int)Math.ceil(currentTime) ;
				}else{//说明小数部分可以省略
					currentEnd = currentStart + freeTime + (int)Math.floor(currentTime) ;
				}
				dayHour = decimal;
				timeArray[i][currentEnd] = 9;		
			}	
			previousStart = currentStart;
			previousEnd = currentEnd;
			previousHour = currentTime;
			previousIsContinuity = IsContinuity;
			flowFlag = TransfersNum == -1?false:true;
			i++;
			j = currentEnd;
		}
 		
// 		for(int x = 1;x<timeArray.length;x++){
// 			for(int y = 1;y<timeArray[x].length;y++){
// 				if(timeArray[x][y] == 0){
// 					System.out.print("0");
// 				}else{
// 					System.out.print("_");
// 				}
// 			}
// 			System.out.println();
// 		}
 		s = j;
		return s;
	}
	
	/**
	 * 流水作业，对工序进行安排时间
	 * @param fProduceOrderID
	 * @param startDate
	 * @param endDate
	 * @param compressionRatio
	 */
	public static void updateProcedurePlan(String fProduceOrderID, String startDate, String endDate, float compressionRatio){
		try{
			SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
			Calendar calendar = Calendar.getInstance();
			Calendar calendar2 = Calendar.getInstance();
			calendar.setTime(format.parse(startDate));
			int[][] procedure = map.get(fProduceOrderID);
			String ksql = "select a.*,SQL.ifnull(a.fTransferNumber,-1) as TransfersNum  from MES_ProcedurePlan a  " +
					"where a.fMainID = '"+fProduceOrderID+"' and a.fIsFinish = '否' order by a.fCode asc ";
			Table table = KSQL.select(ksql, null, DATA_MODEL, null);
			Iterator<Row> rows = table.iterator();
			int i =1;//数组下标
			calendar2.setTime(format.parse(startDate));
			int previous = 0;
			Date previousStartDate = null;
			int num = -1;//记录上一道工序是否流水作业的标识
			while(rows.hasNext()){
				Row row = rows.next();
				String ID = row.getString("a");
				String IsContinuity = row.getString("fIsContinuity");//是否连续标识
				calendar.setTime(format.parse(startDate));
				int j = 1;//数组下标
				for(; j < procedure[i].length ;j++)
					if(procedure[i][j] == 9) break;
				
				calendar.add(Calendar.DAY_OF_YEAR, j-1);
				Date sDate = calendar.getTime();
				if(num > 0 ){ /*&& calendar2.getTime().after(sDate) */
					//如果上一道工序为流水作业工序，则需要计算当前工序的开始时间
					//计算方式：上一道工序的开始时间 + 相差得天数 = 当前开始时间
					calendar.setTime(previousStartDate);
					calendar.add(Calendar.DAY_OF_YEAR, j-previous);
					sDate = calendar.getTime();
					calendar2.setTime(sDate);
				}else{
					sDate = calendar2.getTime();
					sDate = calendar.getTime();
					calendar2.setTime(sDate);
				}	
				previous = j;
				num = row.getInt("TransfersNum");
				int h = j + 1;
				boolean flag = true;
				for(;h < procedure[i].length;h++){
					if(procedure[i][h] == 9) {flag = false;break;}
				}
				int StdLeadtime = h - j + 1;//标准提前期
				if(flag){
					StdLeadtime = 1;
				}
				float PlanLeadtime = (float)Math.round((StdLeadtime * compressionRatio));//计划提前期
				
				calendar.add(Calendar.DAY_OF_YEAR, (int)PlanLeadtime-1);
				
				//判断当天是否是休息日
				Calendar tempCal = Calendar.getInstance();
				tempCal.setTime(sDate);
				while(true){
					
					ksql = " select a.* from BAS_CalendarList a optional join BAS_Calendar b on b = a.fMainID " +
							"where b.fIsDefaultCalendar = '是' and a.fDate = SQL.str_to_date('" + format.format(tempCal.getTime()) + "','%Y-%m-%d') " +
									"and a.fIsWorkingDay = '是'   " ;
					Table cT = KSQL.select(ksql, null, DATA_MODEL, null);
					if(cT.size() > 0){
						sDate = tempCal.getTime();
						calendar2.setTime(sDate);
						break;
					}else{
						tempCal.add(Calendar.DAY_OF_YEAR, 1);
					}
				}
				
				//需要加上休息日的天数
				int restDay = 0;
				if(num < 0){
					ksql = "select a.* from BAS_CalendarList a optional join BAS_Calendar b on b = a.fMainID " +
							"where b.fIsDefaultCalendar = '是'  " +
							"and a.fDate >= SQL.str_to_date('" + format.format(sDate) + "','%Y-%m-%d') " +
									"order by a.fDate asc limit 0,"+(int)PlanLeadtime+"  ";
					Table dayTable = KSQL.select(ksql, null, DATA_MODEL, null);
					Iterator<Row> dayTableRows = dayTable.iterator();
					while(dayTableRows.hasNext()){
						Row dayTableRow = dayTableRows.next();
						if("否".equals(dayTableRow.getString("fIsWorkingDay"))){
							restDay++;
						}
					}
				}
				calendar2.add(Calendar.DAY_OF_YEAR, (int)PlanLeadtime + restDay - 1);
				Date eDate = calendar2.getTime();
				previousStartDate = sDate;
				ksql = "update MES_ProcedurePlan a set a.fPlannedStartDate = SQL.str_to_date('" + format.format(sDate) + "','%Y-%m-%d')," +
						"a.fPlannedEndDate = SQL.str_to_date('" + format.format(eDate) + "','%Y-%m-%d')," +
						"a.fPlannedCycle = " + PlanLeadtime + ",a.fRatio = " + compressionRatio + " where a = '"+ID+"' ";
				KSQL.executeUpdate(ksql, null, DATA_MODEL, null);
				i++;
				if("是".equals(IsContinuity)){
					calendar2.setTime(sDate);
				}else{
					calendar2.add(Calendar.DAY_OF_YEAR, 1);
				}
				
			}
		}catch(Exception e){
			e.printStackTrace();
			throw new RuntimeException();
		}
		
	}
	
	/**
	 * 
	 * @Title: orderMinTime
	 * @Description: TODO(以工时计算最短时间)
	 * @param orderID			生产订单ID
	 * @param resultNum			投产数量
	 * @return    
	 * 参数
	 * @author ChenXiaoChen
	 * @date 2019-7-10
	 * @return int    返回类型
	 *
	 */
	public static int orderMinTime(String orderID,int resultNum){
		int min = 0;
		try{
			String ksql = "select a.*,b.fClassesNumber,b.fAfterDay from MES_ProcedurePlan a optional join BAS_WorkCenter b on b = a.fWorkCenterID " +
					"where  a.fMainID = '" + orderID + "'  ";//and a.fFinishNumber < "+resultNum+"
			Table t = KSQL.select(ksql, null, DATA_MODEL, null);
			Iterator<Row> rows = t.iterator();
			int s = 0;
			while (rows.hasNext()) {
				Row row = rows.next();
				int FinishNumber = row.getInt("fFinishNumber");//完工数量
				int time = row.getInt("fStandardCycle");// 工序的标准提前期
				int EachGroupNumber = row.getInt("fEachGroupNumber");//每组数量
				int ClassesNumber = row.getInt("fClassesNumber");//班次数
				int OutToNumber = row.getInt("fOutToNumber");//工艺周期
				//去掉工作中心的前后等待时间和准结时间，计算此工序最小使用时间
				time = ((resultNum - FinishNumber)/EachGroupNumber)/ClassesNumber + OutToNumber;
				s = s + time;
			}
			return min;
		}catch(Exception e){
			e.printStackTrace();
			throw new RuntimeException();
		}
	}
	
	/**
	 * 计算日期是否要朝前或朝后移
	 * 
	 * @param calendar
	 *            日期参数
	 * @param state
	 *            开始日期/结束日期
	 * @return
	 */
	public static String calculateWorkingDay(Calendar calendar, String state) {
		try {
			if (!judgeIsWorkingDay(calendar)) {
				if (state.equals("startDate")) {
					while (!judgeIsWorkingDay(calendar)) {
						calendar.add(Calendar.DATE, 1);
					}
					return new SimpleDateFormat("yyyy-MM-dd").format(calendar.getTime());
				} else {
					while (!judgeIsWorkingDay(calendar)) {
						calendar.add(Calendar.DATE, -1);
					}
					return new SimpleDateFormat("yyyy-MM-dd").format(calendar.getTime());
				}
			}
		} catch (Exception e) {
			throw new RuntimeException(e.getMessage());
		}
		return new SimpleDateFormat("yyyy-MM-dd").format(calendar.getTime());
	}
	
	/**
	 * 更新工序计划(新版  按小时算)
	 * 
	 * @param fProduceOrderID
	 *            生产订单ID
	 * @param startDate
	 *            开始日期
	 * @param endDate
	 *            结束日期
	 * @param compressionRatio
	 *            压缩比
	 */
	public static void updateProcedurePlan(String fProduceOrderID, String startDate, String endDate, BigDecimal compressionRatio, int level) {
		try {

			// 确认生产订单的计划开始日期和计划结束日期之后，再将时间按每道工序展开
			// 展开方式：每道工序的标准提前期乘上压缩比算出压缩后的天数后，再将这些天数匹配默认日历算出每道工序的计划开工日期以及计划完工日期
			SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
			SimpleDateFormat timeFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm");
			Calendar calendar = Calendar.getInstance();
			calendar.setTime(format.parse(startDate));
			calendar.set(calendar.HOUR_OF_DAY, 8);
			Date planendtime = null;
			String ksql = "select a.*,c.fDocumentNumber,f.fClassesNumber from MES_ProcedurePlan a  "
					+ "optional join MES_ProductionOrder c on c = a.fMainID optional join BAS_WorkCenter f on f = a.fWorkCenterID " +
					"where a.fMainID = '"+ fProduceOrderID+ "'   order by a.fCode asc";
			Table table = KSQL.select(ksql, null, DATA_MODEL, null);
			if (table.size() > 0) {
				Iterator<Row> rows = table.iterator();
				while (rows.hasNext()) {
					boolean nextDay = false;//是否切换到下个工作日
					Row row = rows.next();
					String id = row.getString("a");// 当前工序计划表ID
					String orderNumber = row.getString("fDocumentNumber");//生产订单号
					int ClassesNumber = row.getInt("fClassesNumber");//班次数
					int pnum = row.getInt("fCode");//工序号
					errormessage = "【" + orderNumber + "】" + "下第   " + pnum + "  工序标准提前期为空，请设置";
					int StandardCycle = row.getInt("fStandardCycle");// 标准提前期
					if (StandardCycle == 0) {
						errormessage = "【" + orderNumber + "】" + "下第   " + pnum + "  工序标准提前期为0，请调整工时定额或工作中心的前后等待时间";
						int i = 1 / 0;
					}
					BigDecimal plantime = new BigDecimal(StandardCycle);
					errormessage = "error";
					Date planstarttime = calendar.getTime();// 计划开始日期

					String starttime = "";
					if(calendar.get(Calendar.HOUR_OF_DAY) < 10){
						starttime = "0" + calendar.get(Calendar.HOUR_OF_DAY)+":";
					}else{
						starttime = calendar.get(Calendar.HOUR_OF_DAY)+":";
					}
					if(calendar.get(Calendar.MINUTE) < 10){
						starttime += "0"+calendar.get(Calendar.MINUTE);
					}else{
						starttime += calendar.get(Calendar.MINUTE);
					}
					
					// 判断提前期是否可压缩
					plantime = new BigDecimal(new BigDecimal(StandardCycle).multiply(compressionRatio).floatValue());//计划提前期；(float)(StandardCycle * compressionRatio)
					int day = 0;//天数
					int hour = 0;//小时
					int min = 0;//分钟
					
					BigDecimal point = new BigDecimal(0);
					//将计划提前期分析
					if(plantime.compareTo(new BigDecimal(1)) == 1){
						day = plantime.ROUND_DOWN;//取出天数
						
					}
							
				//	BigDecimal planLead = (plantime/StandardCycle);//压缩比
					
					float hours = (plantime.subtract(new BigDecimal(day)).floatValue()) * (8*ClassesNumber) + day * (8*ClassesNumber);//总小时数
					hour = (int)(Math.floor(hours));
					
					if(calendar.get(calendar.HOUR_OF_DAY) >= (8 + 8*ClassesNumber) ){
						//将时间调整到下一个工作日的 8 am
						ksql = "select p.fDate from BAS_CalendarList p where SQL.date(p.fDate)> SQL.date(stringToDate('"
								+ format.format(calendar.getTime())	+ "')) "
								+ "and p.fIsWorkingDay = '是' and p.fMainID = (select q from BAS_Calendar q where q.fIsDefaultCalendar = '是')  " 
								+"order by p.fDate asc limit 0,1 ";
						Table c = KSQL.select(ksql, null, DATA_MODEL, null);
						if(c.size() == 0){
							errormessage = "运算结果超出默认日历，请联系管理员增加默认日历的年份至" + calendar.get(calendar.YEAR) + "年!";
						}
						calendar.setTime(c.iterator().next().getDate("fDate"));
						calendar.set(Calendar.HOUR_OF_DAY, 8);
						if(calendar.get(Calendar.HOUR_OF_DAY) < 10){
							starttime = "0" + calendar.get(Calendar.HOUR_OF_DAY)+":";
						}else{
							starttime = calendar.get(Calendar.HOUR_OF_DAY)+":";
						}
						if(calendar.get(Calendar.MINUTE) < 10){
							starttime += "0"+calendar.get(Calendar.MINUTE);
						}else{
							starttime += calendar.get(Calendar.MINUTE);
						}
						planstarttime = calendar.getTime();// 计划开始日期
					}
					
					//根据小时往后推，直至消耗完为止
					while(hour > 0){
						int currentHour = calendar.get(calendar.HOUR_OF_DAY);
						if( (currentHour + hour) > (8 + 8*ClassesNumber) ){//如果当前小时数加上当前小时大于 今天下班时间   说明要排到第二天
							int value = (8 + 8*ClassesNumber) - currentHour;
							hour = hour - value;
							
							if(ClassesNumber == 3){
								calendar.add(Calendar.HOUR_OF_DAY, value);
							}else{
								//将时间调整到下一个工作日的 8 am
								ksql = "select p.fDate from BAS_CalendarList p where SQL.date(p.fDate)> SQL.date(stringToDate('"
										+ format.format(calendar.getTime())	+ "')) "
										+ "and p.fIsWorkingDay = '是' and p.fMainID = (select q from BAS_Calendar q where q.fIsDefaultCalendar = '是')  " 
										+"order by p.fDate asc limit 0,1 ";
								Table c = KSQL.select(ksql, null, DATA_MODEL, null);
								if(c.size() == 0){
									errormessage = "运算结果超出默认日历，请联系管理员增加默认日历的年份至" + calendar.get(calendar.YEAR) + "年!";
								}
								calendar.setTime(c.iterator().next().getDate("fDate"));
								calendar.set(Calendar.HOUR_OF_DAY, 8);
							}
							
							
						}else if( (currentHour + hour) == (8 + 8*ClassesNumber) ){ //说明当天正好排完
							hour = 0;//全部排完
							calendar.add(Calendar.HOUR_OF_DAY, hour);
							if(ClassesNumber != 3){//如果是3班，正好排完则不需要调整时间
								nextDay = true;
							}
							
						}else{//说明可以直接排进来
							calendar.add(calendar.HOUR_OF_DAY, hour);	
							hour = 0;//全部排完
						}
					}
					String endtime = "";
					if(calendar.get(Calendar.HOUR_OF_DAY) < 10){
						endtime = "0" + calendar.get(Calendar.HOUR_OF_DAY)+":";
					}else{
						endtime = calendar.get(Calendar.HOUR_OF_DAY)+":";
					}
					if(calendar.get(Calendar.MINUTE) < 10){
						endtime += "0"+calendar.get(Calendar.MINUTE);
					}else{
						endtime += calendar.get(Calendar.MINUTE);
					}
					
					// 更新当前工序的计划开始日期以及计划完工日期
//					ksql = "update MES_ProcedurePlan a set a.fPlannedStartDate = SQL.date('" + format.format(planstarttime) + "')," +
//							"a.fPlannedEndDate = SQL.date('" + format.format(calendar.getTime())+ "') ,a.fPlanLeadtime = " + plantime + "," +
//							"a.fRatio = " + plantime/StandardCycle + ",a.fBeginTime = '"+starttime+"',a.fEndTime = '"+endtime+"' where a = '" + id + "'";
					ksql = "update MES_ProcedurePlan a set a.fPlannedStartDate = SQL.date('" + format.format(planstarttime) + "')," +
							"a.fPlannedEndDate = SQL.date('" + format.format(calendar.getTime())+ "') ,a.fPlannedCycle = " + plantime + "," +
							"a.fRatio = " + compressionRatio + " where a = '" + id + "'";
					KSQL.executeUpdate(ksql, null, DATA_MODEL, null);
					
					if(nextDay){
						//将时间调整到下一个工作日的 8 am
						ksql = "select p.fDate from BAS_CalendarList p where SQL.date(p.fDate)> SQL.date(stringToDate('"
								+ format.format(calendar.getTime())	+ "')) "
								+ "and p.fIsWorkingDay = '是' and p.fMainID = (select q from BAS_Calendar q where q.fIsDefaultCalendar = '是')  " 
								+"order by p.fDate asc limit 0,1 ";
						Table c = KSQL.select(ksql, null, DATA_MODEL, null);
						if(c.size() == 0){
							errormessage = "运算结果超出默认日历，请联系管理员增加默认日历的年份至" + calendar.get(calendar.YEAR) + "年!";
						}
						calendar.setTime(c.iterator().next().getDate("fDate"));
						calendar.set(Calendar.HOUR_OF_DAY, 8);
					}
					
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException(errormessage);
		}
	}
	
	/**
	 * 深度优先搜索
	 * 
	 * @param root
	 *            根节点
	 * @param sum
	 *            根节点时长
	 * @param tree
	 *            父子树
	 * @param path
	 *            当前路径
	 * @param length
	 *            订单时长
	 * @param limit
	 *            成品最长时间
	 * @param compressed
	 *            是否关键路径
	 * @param min_length
	 *            最短时长
	 * @param most_compressed_path
	 *            可压缩路径
	 */
	public static void dfs(int root, double sum, int[][] tree, int path[], double length[], int limit, boolean compressed[], double[] min_length, int[] most_compressed_path) {
		if (tree[root][1] == 0) { // 如果为叶子节点
			if (sum - limit < 0.000001)
				return;

			if (sum > longest)
				longest = sum;
			to_compress = 0;
			all_fixed = true;

			for (int i = 1; i <= count; i++) {
				if (!compressed[path[i]]) {
					if (length[path[i]] > min_length[path[i]])
						all_fixed = false;
					to_compress += length[path[i]] - min_length[path[i]];
				}
			}
			if (!all_fixed && to_compress - sum + limit == 0) {
				unable_to_finish = true;
				min_rate = 0;
				return;
			}
			temp_rate = (to_compress - sum + limit) / to_compress;
			if (temp_rate < 0) {
				unable_to_finish = true;
				return;
			}
			if (temp_rate < min_rate) {
				min_rate = temp_rate;
				most_compressed_count = count;
				for (int i = 1; i <= count; i++)
					most_compressed_path[i] = path[i];
			}
			return;
		}

		// 遍历订单树
		count++;
		int i = 1;
		while (tree[root][i] != 0) {
			path[count] = tree[root][i];
			dfs(tree[root][i], sum + length[tree[root][i]], tree, path, length, limit, compressed, min_length, most_compressed_path);
			i++;
		}
		count--;
	}

	/**
	 * 遍历资源链路
	 * 
	 * @param fParent
	 *            父ID
	 * @param fatherIndex
	 *            父序号
	 * @param father
	 *            父序号数组
	 * @param length
	 *            工作时长数组
	 * @param original_length
	 *            最初工作时长数组
	 * @param produceOrderID
	 *            订单ID数组
	 * @param tree
	 *            父子关系二维数组
	 * @param min_length
	 *            最短工作时长数组
	 */
	public static void traverseResoucesChain(String fParent, int fatherIndex, int father[], double length[], double original_length[], String[] produceOrderID, int tree[][], double[] min_length) {
		try {
			String ksql = "select p, p.fRelevantID, p.fStockCode,p.fMaterialSource from MES_TreeOrder p where p.fParent = '" + fParent + "'"; // 找出父节点下的子节点

			Table table = KSQL.select(ksql, null, DATA_MODEL, null);
			if (table.size() > 0) { // 遍历子节点
				Iterator<Row> rows = table.iterator();
				int i = 1;
				while (rows.hasNext()) {
					Row row = rows.next();
					if (row.getString("fMaterialSource").equals("生产")) {
						ksql = "select p, p.fDocumentNumber,p.fNumber , p.fStandardCycle from MES_ProductionOrder p where p = '" + row.getString("fRelevantID")
								+ "' and p.fOrderStatus <> '结令'"; // 根据单据编号到生产订单中查询
						Table table2 = KSQL.select(ksql, null, DATA_MODEL, null);
						if (table2.size() > 0) { // 如果能找到，说明绑定的为生产订单
							Row row2 = table2.iterator().next();
							errormessage = "【" + row2.getString("fDocumentNumber") + "】订单没有标准提前期!";
							if (row2.getInt("fStandardCycle") != 0) { // 如果订单的标准提前期不为0
								father[++index] = fatherIndex; // 记录当前订单的父订单
								length[index] = row2.getInt("fStandardCycle"); // 给订单时长赋值
								String orderID = row2.getString("p");//生产订单ID
								int orderNumber = row2.getInt("fNumber");//生产订单数量
								original_length[index] = length[index]; // 给最初时长赋值
								min_length[index] = orderMinTime(orderID, orderNumber); // 给最小时长赋值
								produceOrderID[index] = row.getString("fRelevantID"); // 给订单ID数组赋值
								tree[fatherIndex][i] = index; // 在二维数组中建立数组关系
								traverseResoucesChain(row.getString("p"), index, father, length, original_length, produceOrderID, tree, min_length);
								i++;
							}
						}

					}
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException(e.getMessage());
		}
	}

	/**
	 * 遍历父子关系二维数组
	 * 
	 * @param farherIndex
	 *            父节点序号
	 * @param fatherStartDate
	 *            父节点开始日期
	 * @param tree
	 *            父子关系二维数组
	 * @param length
	 *            工作时长数组
	 * @param rate_
	 *            压缩比数组
	 * @param produceOrderID
	 *            生产订单ID数组
	 */
	public static void traverseTree(int farherIndex, String fatherStartDate, int[][] tree, double[] length, String[] produceOrderID, double[] original_length, double[] min_length) {
		int j = 1;
		while (tree[farherIndex][j] != 0) {
			if (length[tree[farherIndex][j]] != min_length[tree[farherIndex][j]]) {
				Calendar calendarEnd = Calendar.getInstance();
				try {
					calendarEnd.setTime(new SimpleDateFormat("yyyy-MM-dd").parse(fatherStartDate));
				} catch (ParseException e) {
					e.printStackTrace();
				}
				calendarEnd.add(Calendar.DATE, -(int) (length[tree[farherIndex][j]] - 1));

				int noWorkingDay = calculateIsNotWorkingDay(new SimpleDateFormat("yyyy-MM-dd").format(calendarEnd.getTime()), fatherStartDate);
				calendarEnd.add(Calendar.DATE, -noWorkingDay);
				
				//算出当前生产订单的标准提前期
				String ksql = "select a.fDocumentNumber,a.fNumber from MES_ProductionOrder a where a = '"+produceOrderID[tree[farherIndex][j]]+"'";
				Table table = KSQL.select(ksql, null, DATA_MODEL, null);
				String orderNo = table.iterator().next().getString("fDocumentNumber");//生产订单号
				int ResultNum = table.iterator().next().getInt("fNumber");//实际投产数量
				float orderSt = orderTime(produceOrderID[tree[farherIndex][j]],orderNo,ResultNum);//标准提前期
				float orderPlan = (float) Math.round(((int) length[tree[farherIndex][j]]/orderSt)*100)/100;//生产订单压缩比
				
				String ksql3 = "";
				if(IsStart(produceOrderID[tree[farherIndex][j]])){//如果已有工序完工，则不修改计划开始日期
					ksql3 = "update MES_ProductionOrder p set p.fRatio = "	+ orderPlan + ",  p.fPlannedEndDate = :toDate('" + calculateWorkingDay(getCalendar(fatherStartDate), "endDate") + "') , p.fPlannedCycle = " + (int) length[tree[farherIndex][j]]
							+ ",p.fStandardCycle = "+orderSt+",p.fSign = 0 where p = '" + produceOrderID[tree[farherIndex][j]] + "'";
				}else{
					//新版
					ksql3 = "update MES_ProductionOrder p set p.fRatio = "	+ orderPlan + ", p.fPlannedStartDate = :toDate('" + calculateWorkingDay(calendarEnd, "startDate")
							+ "')" + " , p.fPlannedEndDate = :toDate('" + calculateWorkingDay(getCalendar(fatherStartDate), "endDate") + "') , p.fPlannedCycle = " + (int) length[tree[farherIndex][j]]
							+ ",p.fStandardCycle = "+orderSt+" where p = '" + produceOrderID[tree[farherIndex][j]] + "'";
				}
				KSQL.executeUpdate(ksql3, null, DATA_MODEL, null);
				isContinuity(produceOrderID[tree[farherIndex][j]]);
				System.out.println("a"+orderSt);
				updateProcedurePlan(produceOrderID[tree[farherIndex][j]], new SimpleDateFormat("yyyy-MM-dd").format(calendarEnd.getTime()), fatherStartDate,new BigDecimal(orderPlan), 1);

				calendarEnd.add(Calendar.DATE, -1);
				traverseTree(tree[farherIndex][j], new SimpleDateFormat("yyyy-MM-dd").format(calendarEnd.getTime()), tree, length, produceOrderID, original_length, min_length);
			}

			j++;
		}
	}
	
	/**
	 * 获得日历实例
	 * 
	 * @param date
	 * @return
	 */
	public static Calendar getCalendar(String date) {
		Calendar calendar = Calendar.getInstance();
		try {
			calendar.setTime(new SimpleDateFormat("yyyy-MM-dd").parse(date));
		} catch (ParseException e) {
			// TODO 自动生成的 catch 块
			e.printStackTrace();
		}
		return calendar;
	}
	
	/**
	 * 
	 * 计算开始日期到结束日期中的非工作日
	 * 
	 * @param beginDate
	 *            开始日期
	 * @param endDate
	 *            结束日期
	 * @return
	 */
	public static int calculateIsNotWorkingDay(String beginDate, String endDate) {
		try {
			String ksql = "select p from BAS_CalendarList p where SQL.date(p.fDate) between '" + beginDate + "' and '" + endDate + "'"
					+ "and p.fIsWorkingDay = '否' and p.fMainID = (select q from BAS_Calendar q where q.fIsDefaultCalendar = '是')";
			Table table = KSQL.select(ksql, null, DATA_MODEL, null);
			if (table.size() > 0) {
				Calendar calendarBegin = Calendar.getInstance();
				calendarBegin.setTime(new SimpleDateFormat("yyyy-MM-dd").parse(beginDate));
				calendarBegin.add(Calendar.DATE, -table.size());

				Calendar calendarEnd = Calendar.getInstance();
				calendarEnd.setTime(new SimpleDateFormat("yyyy-MM-dd").parse(beginDate));
				calendarEnd.add(Calendar.DATE, -1);
				return table.size() + calculateIsNotWorkingDay(new SimpleDateFormat("yyyy-MM-dd").format(calendarBegin.getTime()), new SimpleDateFormat("yyyy-MM-dd").format(calendarEnd.getTime()));
			}
		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException(e.getMessage());
		}
		return 0;
	}
	
	/**
	 * 判断是否是工作日
	 * 
	 * @param calendar
	 *            日期参数
	 * @return
	 */
	public static boolean judgeIsWorkingDay(Calendar calendar) {
		try {
			String currentDate = new SimpleDateFormat("yyyy-MM-dd").format(calendar.getTime());
			String ksql = "select p from BAS_CalendarList p where p.fDate = :toDate('" + currentDate
					+ "') and p.fIsWorkingDay = '否' and p.fMainID = (select q from BAS_Calendar q where q.fIsDefaultCalendar = '是')";
			Table table = KSQL.select(ksql, null, DATA_MODEL, null);
			if (table.size() > 0) {
				return false;
			}
		} catch (Exception e) {
			throw new RuntimeException(e.getMessage());
		}
		return true;
	}
	
	/**
	 * continuity
	 * @Title: isContinuity
	 * @Description: TODO(对此订单下的工序进行处理，判断是否为连续工序)
	 * @param orderID    生产订单ID
	 * @author ChenXiaoChen
	 * @date 2018-12-20
	 * @return void    返回类型
	 *
	 */
	public static void isContinuity(String orderID){
		String ksql = "select a.*,b.fColdOrHot from MES_ProcedurePlan a optional join BAS_WorkCenter b on b = a.fWorkCenterID " +
				" where a.fMainID = '"+orderID+"'  order by  a.fCode asc ";
		Table table = KSQL.select(ksql, null, DATA_MODEL, null);
		Iterator<Row> rows = table.iterator();
		boolean flag = true;
		while(rows.hasNext()){
			flag = false;
			Row row = rows.next();
			String workCenterID = row.getString("fWorkCenterID");//工作中心
			String currentID = row.getString("a");//当前行ID
			String ColdOrHot = row.getString("fColdOrHot");//冷线还是热线		
			int Sequence = row.getInt("fCode");//工序号
			ksql = "select a.fWorkCenterID from MES_ProcedurePlan a  " +
					"where a.fMainID = '"+orderID+"' and a.fCode > "+Sequence+" order by  a.fCode asc limit 0,1";
			Table nextTable = KSQL.select(ksql, null, DATA_MODEL, null);
			if(nextTable.size() > 0){
				String lastID = nextTable.iterator().next().getString("fWorkCenterID");//上道工作中心ID
				if(lastID.equals(workCenterID)){//如果查不到上一道，说明是第一道工序，是需要考核的
					flag = true;
				}else{//拿当前的和下一道比
					flag = false;
				}
			}
			if(flag && "热线".equals(ColdOrHot) ){
				ksql = "update MES_ProcedurePlan a set a.fIsContinuity = '是' where a = '"+currentID+"' ";
				KSQL.executeUpdate(ksql, null, DATA_MODEL, null);
			}else{
				ksql = "update MES_ProcedurePlan a set a.fIsContinuity = '否' where a = '"+currentID+"' ";
				KSQL.executeUpdate(ksql, null, DATA_MODEL, null);
			}
		}
	}
	
	/**
	 * continuity
	 * @Title: isContinuity
	 * @Description: TODO(下发功能)
	 * @param orderID    生产订单ID
	 * @author ChenXiaoChen
	 * @date 2018-12-20
	 * @return void    返回类型
	 *
	 */
	public static void orderLowHair(){
		
	}
}