import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.UUID;

import com.Resource;
import com.ResourceControlCenter;
import com.Sql;
import com.tool;
import com.alibaba.fastjson.JSONObject;
import com.google.protobuf.TextFormat.ParseException;
import com.justep.system.context.ContextHelper;
import com.justep.system.data.KSQL;
import com.justep.system.data.Row;
import com.justep.system.data.SQL;
import com.justep.system.data.Table;


public class Calculation {
	/**此java文件公用全局变量**/
	/******************************************************************************************************************/	
	/***/private static final String DATA_MODEL = "/DB/table/data";//表的模块									   /***/
	/***/private static final String DATA_MODELOFK3 = "/DB/kingdeeK3/data";//k3数据表的模块						   /***/
	/***/private static String ERRORMESSAGE = "";//错误信息														   /***/
	/***/private static String ERRORSTATUS = "ok";//错误标识     默认“ok”，如果运算过程中出现错误，则将此改为error		   /***/
	/***/private static String personName = ContextHelper.getPersonMember().getName();//当前登陆人名称			   /***/
	/***/private static String personFID = ContextHelper.getPersonMember().getID();//当前登陆人fID	
	/***/private static String personID = personFID.substring(0, personFID.indexOf("@"));//当前登陆人ID			   /***/
	/******************************************************************************************************************/
		
	 
	private static String BookID = null;//生产任务书ID   MRP运算时使用
	/**
	 * 
	 * @Title: MRPOperation
	 * @Description: 对生产任务书进行MRP运算
	 * @param taskBookID		   生产任务书主表ID
	 * @param IsStrictImplement  是否考虑共模工序
	 * @author ChenXiaoChen
	 * @date 2018-9-27
	 * @return void    返回类型
	 *
	 */
	public static JSONObject MRPOperation(String taskBookID,String IsStrictImplement){
		/*
		 * 1.检查之前是否运算过，如果运算过，需要将原有的MRP运算记录清除，并解锁资源
		 * 2.对此生产任务书下的明细，进行BOM检查，如果此下子件BOM没有全部有效，则提示需要做BOM
		 * 3.根据BOM进行展开，展开计算计划用量，父级数量，并形成结构树，记录在MRp记录表中
		 *   每级节点计算后，对于生产任务书 任务类型 fTaskType 为订单的，需要利库存，利预投
		 * 	   对于任务类型为预投的，除顶级件外，需要利库存，不利预投
		 * 4.利库存以及利预投的同时，需要进行资源锁定
		 */
		JSONObject result = new JSONObject();//作为返回值用
		try{
			BookID = taskBookID;//将生产任务书的ID赋给全局变量，便于后续调用子方法时使用
			
			//检查BOM完整性
			//eachBook("check",null);
			
			if("ok".equals(ERRORSTATUS)){
				//计算前先检查是否已有MRP运算记录
				beforOperation();
				
				//根据bom展开进行MRP运算
				eachBook("calculation","订单");
				
				//将生产任务书的状态置为已运算
				String sql = "update MES_BookOfManage a set a.fStatus = '已运算',a.version = version + 1 where a.fID = '"+taskBookID+"' ";
				SQL.executeUpdate(Sql.DBMySql(sql), null, DATA_MODEL);
			}
				
				
			result.put("status", ERRORSTATUS);
			result.put("message", ERRORMESSAGE);
			return result;
		}catch(Exception e){
			e.printStackTrace();
			throw new RuntimeException();
		}
			
	}
	
	/**
	 * 
	 * @Title: beforOperation
	 * @Description: 进行MRP运算前，将之前运算的数据清空，以便于能够反复运算(待完善)
	 * @    参数
	 * @author ChenXiaoChen
	 * @date 2018-10-18
	 * @return void    返回类型
	 *
	 */
	public static void beforOperation(){
		//计算开始前处理
		//查找是否已经有MRP记录
		String sql = "select a.fid from MES_MRPResult a where a.fTaskBookID = '"+BookID+"'";
		Table countTable = SQL.select(Sql.DBMySql(sql),null,DATA_MODEL);
		if(countTable.size() > 0){
			//如果已有记录，查找是否有资源占用，如果有，则依次解锁，如果没有，则跳过此梳理步骤
			sql = "select a.* from MES_MRPResult a where a.fTaskBookID = '"+BookID+"' and (a.fInventory > 0 or a.fPrecastNumber > 0 ) ";
			Table table = SQL.select(Sql.DBMySql(sql),null,DATA_MODEL);
			if(table.size() > 0){
				Iterator<Row> rows = table.iterator();    
				while(rows.hasNext()){
					Row row = rows.next();
					String ListID = row.getString("fListID");//生产任务书明细ID
					String code = row.getString("fStockCode");//物料编码
					ResourceControlCenter.unlockAllResourcesOfDemand(code, ListID, "生产任务书");
				}
			}
			
			//所有处理完成后，将MRP记录清除
			sql = "delete from MES_MRPResult  where fTaskBookID = '"+BookID+"' ";
			SQL.executeUpdate(Sql.DBMySql(sql), null, DATA_MODEL);	
		}	
	}
		
	/**
	 * 
	 * @Title: eachBook
	 * @Description: 根据生产任务书ID遍历下方所有零件，根据不同的type类别来进行不同的操作处理
	 * @param taskBookID		生产任务书ID
	 * @param action			动作，“check”：检查bom   “calculation”：进行MRP计算
	 * @param type    参数		生产任务书的类型    预投  /   订单
	 * @author ChenXiaoChen
	 * @date 2018-9-29
	 * @return void    返回类型
	 *
	 */
	public static void eachBook(String action,String type){
		try{
			//找出此任务书下的所有投产的零件
			String sql = "select a.* from MES_BookOfManageList a where a.fMainID = '"+BookID+"' ";
			Table table = SQL.select(Sql.DBMySql(sql),null,DATA_MODEL);
			Iterator<Row> rows = table.iterator();    
			while(rows.hasNext()){
				Row row = rows.next();
				String StockCode = row.getString("fStockCode");//物料编码
				String currentID = row.getString("fID");//生产任务书明细ID
				int number = row.getInt("fNumber");//投产的数量
				Date endDate = row.getDate("fDeliveryDate");//交货期
				int Sign = row.getInt("fSign");//标识，0 代表需要投产，1代表是因为共模所需要添加投产零件
				String CommonModeSign = row.getString("fCommonModeSign");//共模标识
				if("check".equals(action)){//调用检查BOM方法进行BOM检查
					openBom(StockCode);
				}else if("calculation".equals(action)){//调用顶级件按MRP运算方法进行运算
					MRP(StockCode,type,number,endDate,currentID,Sign,CommonModeSign);
				}
			}
			
			if("calculation".equals(action)){//如果是进行MRP运算，则需要对运算的结果进行比对处理(主要处理投产数量，需要符合模具使用性)
				finalNumber();
			}
		}catch(Exception e){
			e.printStackTrace();
			throw new RuntimeException();
		}		
	}
		
	/**
	 * 
	 * @Title: openBom
	 * @Description: 根据BOM展开，核对此零件下的子项BOM是否完备，是否是启用状态
	 * @@param stockCode    参数			零件号
	 * @author ChenXiaoChen
	 * @date 2018-9-29
	 * @return void    返回类型
	 *
	 */
	public static void openBom(String stockCode){
		try{
			String sql = "select a.*,b.fStatus,b.fEnabled from CAPP_BOMList a left join CAPP_BOM b on a.fMainID = b.fID " +
					"where b.fStockCode = '"+stockCode+"' and (a.fMaterialSource = '生产' or a.fMaterialSource = '配方')";
			Table table = SQL.select(Sql.DBMySql(sql), null, DATA_MODEL);
			if(table.size() > 0){
				Iterator<Row> rows = table.iterator();    
				while(rows.hasNext()){
					Row row = rows.next();
					String StockCode = row.getString("fStockCode");//物料编码
					String Status = row.getString("fStatus");//BOM状态
					int Enabled = row.getInteger("fEnabled");//是否启用
					if(Status == null){
						ERRORMESSAGE = StockCode+"编码的BOM没有制作,请联系工艺人员编制此编码的BOM!";
						ERRORSTATUS = "error";
					}
					if(Enabled == 0){
						ERRORMESSAGE = StockCode+"编码的BOM不是启用状态,请联系工艺人员将此编码的BOM启用!";
						ERRORSTATUS = "error";
					}
					openBom(StockCode);
				}
			}
		}catch(Exception e){
			e.printStackTrace();
			throw new RuntimeException();
		}
	}
	
	/**
	 * 
	 * @Title: finalNumber
	 * @Description: 对于已经找到资源的产品，并根据模具使用性，计算出每个产品的最终量
	 * @    参数
	 * @author ChenXiaoChen
	 * @date 2018-10-18
	 * @return void    返回类型
	 *
	 */
	public static void finalNumber(){
		try{
			boolean flag = true;
			String sql = "select a.*,ifnull(e.fOutputRate,1 ) as OutputRate,(select ifnull(count(b.fid),1) from MES_MRPResult b where b.fCommonModeSign = a.fCommonModeSign and b.fTaskBookID = '"+BookID+"') as count," +
					"ifnull(c.fRationNum,0) as  RationNum " +
					"from MES_MRPResult a left join CAPP_MouldProductList c on c.fStockCode = a.fStockCode " +
					"left join CAPP_Mould d on c.fMainID = d.fID " +
					"left join BAS_StockArchives e on e.fStockCode = a.fStockCode " +
					"where a.fTaskBookID = '"+BookID+"' and a.fSign = 0 " +
					"order by a.fCommonModeSign,a.fDeliveryDate,a.fSign asc";
			Table result = SQL.select(Sql.DBMySql(sql), null, DATA_MODEL);
			Iterator<Row> rows = result.iterator();    
			while(rows.hasNext()){
				Row row = rows.next();
				String currentID = row.getString("fID");//当前行ID
				String StockCode = row.getString("fStockCode");//物料编码
				int count = row.getInt("count");//共模的产品
				int Requirement = row.getDecimal("fRequirement").intValue();//需求量
				int Quota = row.getInt("RationNum");//一个模具能做出此产品的数量
				String CommonModeSign = row.getString("fCommonModeSign");//共模标识
				BigDecimal OutputRate = row.getDecimal("OutputRate");//合格率
				
				int FinalProductionNumber = 0;//最终投产数
				
				if(count == 1 && Quota != 0){//如果共模的产品只有一个，则只需要比对此产品与模具数量的比值
					BigDecimal x = new BigDecimal(Requirement).divide(OutputRate,0,BigDecimal.ROUND_HALF_UP);
			    	BigDecimal y = new BigDecimal(Quota);
			    	String z = x.divide(y,2,BigDecimal.ROUND_HALF_UP).toString();
			    	if(tool.isInteger(z)){
			    		FinalProductionNumber = Requirement/Quota * Quota;
			    	}else{		    		
			    		FinalProductionNumber = (Requirement/Quota + 1) * Quota;
			    	}
			    	//FinalProductionNumber = new BigDecimal(FinalProductionNumber).divide(OutputRate,0,BigDecimal.ROUND_HALF_UP).intValue();
			    	sql = "update MES_MRPResult  set fFinalNum = "+FinalProductionNumber+" ,fMainPartNumber = '是' where fID = '"+currentID+"' ";
					SQL.executeUpdate(Sql.DBMySql(sql), null, DATA_MODEL);
					flag = true;
				}else if(count == 1 && Quota == 0){//说明此产品没有模具使用性,需求多少生产多少
					FinalProductionNumber = Requirement;
					FinalProductionNumber = new BigDecimal(FinalProductionNumber).divide(OutputRate,0,BigDecimal.ROUND_HALF_UP).intValue();
					sql = "update MES_MRPResult  set fFinalNum = "+FinalProductionNumber+" ,fMainPartNumber = '是' where fID = '"+currentID+"' ";
					SQL.executeUpdate(Sql.DBMySql(sql), null, DATA_MODEL);
					flag = true;
				}else if(count > 1){//当此产品与其他产品共模下单时，需要考虑产品的配比以及与模具的配比
					sql = "select a.*, a.fRequirement/c.fRationNum as mouldNumber  " +
							"from MES_MRPResult a left join CAPP_MouldProductList c on c.fStockCode = a.fStockCode " +
							"left join CAPP_Mould d on c.fMainID = d.fID  " +
							"where a.fTaskBookID = '"+BookID+"' and a.fCommonModeSign = '"+CommonModeSign+"' " +
									"order by mouldNumber desc limit 0,1 ";
					Table mould = SQL.select(Sql.DBMySql(sql), null, DATA_MODEL);
					BigDecimal maxMouldNum = mould.iterator().next().getDecimal("mouldNumber");//模具的最大用量
					maxMouldNum = maxMouldNum.divide(OutputRate,0,BigDecimal.ROUND_HALF_UP);
					int MouldNum = 0;//使用的模具数量
					if(tool.isInteger(maxMouldNum.toString())){
						MouldNum = maxMouldNum.intValue();
			    	}else{
			    		MouldNum = maxMouldNum.intValue() + 1;
			    	}
					MouldNum = maxMouldNum.intValue();
					FinalProductionNumber = MouldNum * Quota;
					//FinalProductionNumber = new BigDecimal(FinalProductionNumber).divide(OutputRate,0,BigDecimal.ROUND_HALF_UP).intValue();
					sql = "update MES_MRPResult a set a.fMainPartNumber = '否', a.fFinalNum = (select b.fRationNum from CAPP_MouldProductList b where b.fStockCode = a.fStockCode ) * "+MouldNum+" " +
							"where a.fTaskBookID = '"+BookID+"' and a.fSign = 1   and a.fCommonModeSign = '"+CommonModeSign+"' ";
					SQL.executeUpdate(Sql.DBMySql(sql), null, DATA_MODEL);
					sql = "select a.fID from MES_MRPResult a where a.fCommonModeSign = '"+CommonModeSign+"' and a.fMainPartNumber = '是' and a.fTaskBookID = '"+BookID+"'";
					Table judge = SQL.select(Sql.DBMySql(sql), null, DATA_MODEL);
					if(judge.size() == 0){
						sql = "update MES_MRPResult  set fFinalNum = "+FinalProductionNumber+" ,fMainPartNumber = '是' where fID = '"+currentID+"' ";
					}else{
						sql = "update MES_MRPResult  set fFinalNum = "+FinalProductionNumber+" ,fMainPartNumber = '否' where fID = '"+currentID+"' ";
					}
					SQL.executeUpdate(Sql.DBMySql(sql), null, DATA_MODEL);
					flag = false;
				}			
			}
			sql = "select a.* from MES_MRPResult a where a.fTaskBookID = '"+BookID+"'  ";
			result = SQL.select(Sql.DBMySql(sql), null, DATA_MODEL);
			rows = result.iterator();  
			while(rows.hasNext()){
				Row row = rows.next();
				String currentID = row.getString("fID");//当前行ID
				String StockCode = row.getString("fStockCode");//物料编码
				BigDecimal FinalNum = row.getDecimal("fFinalNum");//最终量
				String MainPartNumber = row.getString("fMainPartNumber");//主件号
				MRPToBom(currentID, StockCode, FinalNum,MainPartNumber);
			}
		}catch(Exception e){
			e.printStackTrace();
			throw new RuntimeException();
		}
	}
	
	/**
	 * 
	 * @Title: MRP
	 * @Description: 对这张生产任务书下每一个零件进行MRP运算
	 * @param StockCode			存货编码
	 * @param type					生产任务书的类别，  “订单” / “预投”
	 * @param number				投产数量
	 * @param endData				交付日期
	 * @param listID    参数		生产任务书子表ID
	 * @author ChenXiaoChen
	 * @date 2018-10-12
	 * @return void    返回类型
	 *
	 */
	public static void MRP(String StockCode,String type,int number,Date endData,String listID,int Sign,String CommonModeSign){
		try{
			String sql = "";
			String id = UUID.randomUUID().toString().replace("-", "");
			int profitPreinvestment = 0;//利预投数
			int profitStockNumber = 0;//利库存数
			int requirement = number;//最终需求量
			if("订单".equals(type)){//如果投产的是订单，则需要进行利库存，利预投
//				if(endData == null){//如果不是顶级件，则利库存利预投
//					//利库存，如果有库存则进行资源锁定
//					int needNumber = number;//需求数量
//					List<Resource> StockList = ResourceControlCenter.searchResource(StockCode, "库存");
//					for(Resource i : StockList){
//						BigDecimal StockNumber = i.getNumber();//可用资源数
//						if(StockNumber.compareTo(new BigDecimal(needNumber)) == -1 ){
//							//说明资源数小于需求数
//							ResourceControlCenter.lockResources(StockCode, listID, "生产任务书", i.getID(), i.getType(),i.getDocumentNum(), StockNumber);
//							profitStockNumber +=  StockNumber.intValue();
//							needNumber -= StockNumber.intValue();
//						}else if(StockNumber.compareTo(new BigDecimal(needNumber)) == 1){
//							//说明资源数大于需求数
//							ResourceControlCenter.lockResources(StockCode, listID, "生产任务书", i.getID(), i.getType(),i.getDocumentNum(), new BigDecimal(number));
//							profitStockNumber += needNumber;
//							needNumber = 0;
//							break;
//						}else{
//							//说明资源数和需求数正好
//							ResourceControlCenter.lockResources(StockCode, listID, "生产任务书", i.getID(), i.getType(),i.getDocumentNum(), StockNumber);
//							profitStockNumber += needNumber;
//							needNumber = 0;
//							break;
//						}
//					}
//				
//					
//					if(needNumber > 0){//说明还需要利预投
//						//利预投，如果有资源则进行资源锁定
//						List<Resource> orderList = ResourceControlCenter.searchResource(StockCode, "预投");
//						for(Resource i : orderList ){
//							BigDecimal orderNumber = i.getNumber();
//							if(orderNumber.intValue() > needNumber){
//								//说明资源数量比需求数量大
//								ResourceControlCenter.lockResources(StockCode, listID, "生产任务书", i.getID(), i.getType(),i.getDocumentNum(), new BigDecimal(needNumber));
//								profitPreinvestment +=  needNumber;
//								needNumber = 0;
//								break;
//							}else if(orderNumber.intValue() < needNumber){
//								//说明资源数量比需求数量小
//								ResourceControlCenter.lockResources(StockCode, listID, "生产任务书", i.getID(), i.getType(),i.getDocumentNum(), orderNumber);
//								profitPreinvestment +=  needNumber;
//								needNumber -= orderNumber.intValue();
//							}else{
//								//说明资源数量与需求数量一样
//								ResourceControlCenter.lockResources(StockCode, listID, "生产任务书", i.getID(), i.getType(),i.getDocumentNum(), new BigDecimal(needNumber));
//								profitPreinvestment +=  needNumber;
//								needNumber = 0;
//								break;
//							}
//						}
//					}
//				}
				
				requirement = number - profitStockNumber - profitPreinvestment;//最终需求量
					
				sql = "insert into MES_MRPResult  (fID,version,fTaskBookID,fStockCode,fRationNum,fPlanNumber,fListID," +
						"fInventory,fPrecastNumber,fRequirement,fCreatePersonID,fCreateDateTime,fLastUpdatePersonID,fLastUpdateDateTime,fSource,fDeliveryDate,fSign,fCommonModeSign) " +
						"values ('"+id+"',0,'"+BookID+"','"+StockCode+"',"+number+","+number+",'"+listID+"',"+profitStockNumber+","+profitPreinvestment+"," +
						""+requirement+",'"+personID+"',now(),'"+personID+"',now(),'生产',date('"+endData+"'),"+Sign+",'"+CommonModeSign+"')";
			}else if("预投".equals(type)){
				requirement = number - profitStockNumber - profitPreinvestment;//最终需求量
				sql = "insert into MES_MRPResult  (fID,version,fTaskBookID,fStockCode,fRationNum,fPlanNumber,fListID," +
						"fInventory,fPrecastNumber,fRequirement,fCreatePersonID,fCreateDateTime,fLastUpdatePersonID,fLastUpdateDateTime,fSource,fDeliveryDate,fSign,fCommonModeSign) " +
						"values ('"+id+"',0,'"+BookID+"','"+StockCode+"',"+number+","+number+",'"+listID+"',"+profitStockNumber+"," +
						""+profitPreinvestment+","+requirement+",'"+personID+"',now(),'"+personID+"',now(),'生产',date('"+endData+"'),"+Sign+",'"+CommonModeSign+"')";
			}
			SQL.executeUpdate(Sql.DBMySql(sql), null, DATA_MODEL);
			//MRPToBom(id,StockCode,new BigDecimal(requirement));
		}catch(Exception e){
			e.printStackTrace();
			throw new RuntimeException();
		}
	}
	
	/**
	 * 
	 * @Title: MRPToBom
	 * @Description: 递归方法，根据BOM展开，运算每一层的零件的需求量
	 * @param parentID			父ID		
	 * @param parentCode		父级编码
	 * @param parentNumber      父级数量
	 * @author ChenXiaoChen
	 * @date 2018-10-13
	 * @return void    返回类型
	 *
	 */
	public static void MRPToBom(String parentID,String parentCode,BigDecimal parentNumber,String MainPartNumber){
		try{
			String sql = "select a.*,c.fStockType from CAPP_BOMList a left join CAPP_BOM b on a.fMainID = b.fid " +
					" left join BAS_StockArchives c on c.fStockCode = a.fStockCode where b.fStockCode = '"+parentCode+"'  ";
			Table table = SQL.select(Sql.DBMySql(sql), null, DATA_MODEL);
			if(table.size() > 0){
				Iterator<Row> rows = table.iterator();    
				while(rows.hasNext()){
					Row row = rows.next();
					String id = UUID.randomUUID().toString().replace("-", "");//32位字符串
					String StockCode = row.getString("fStockCode");//物料编码
					BigDecimal RationNum = row.getDecimal("fRationNum");//定额用量
					BigDecimal planNum = RationNum.multiply(parentNumber);//计划用量
					BigDecimal attritionRate = row.getDecimal("fYield");//工艺出品率（耗损率）
					String StockType = row.getString("fStockType");//存货类别
					if("配方".equals(StockType) || "是".equals(MainPartNumber) || ("是".equals(MainPartNumber) && !"模具".equals(StockType) )){
						String Source = row.getString("fMaterialSource");//物料来源
						//利库存
						BigDecimal profitStockNumber = new BigDecimal(0);//利库存数
						//利预投
						BigDecimal profitPreinvestment = new BigDecimal(0);//利预投数
						if("生产".equals(Source)){
							//利库存
							int needNumber = planNum.intValue();
							List<Resource> StockList = ResourceControlCenter.searchResource(StockCode, "库存");
							for(Resource i : StockList){
								BigDecimal StockNumber = i.getNumber();//可用资源数
								if(StockNumber.compareTo(new BigDecimal(needNumber)) == -1 ){
									//说明资源数小于需求数
									ResourceControlCenter.lockResources(StockCode, id, "生产任务书", i.getID(), i.getType(),i.getDocumentNum(), StockNumber);
									profitStockNumber = profitStockNumber.add(StockNumber);
									needNumber -= StockNumber.intValue();
								}else if(StockNumber.compareTo(new BigDecimal(needNumber)) == 1){
									//说明资源数大于需求数
									ResourceControlCenter.lockResources(StockCode, id, "生产任务书", i.getID(), i.getType(),i.getDocumentNum(), new BigDecimal(needNumber));
									profitStockNumber = profitStockNumber.add(new BigDecimal(needNumber));
									needNumber = 0;
									break;
								}else{
									//说明资源数和需求数正好
									ResourceControlCenter.lockResources(StockCode, id, "生产任务书", i.getID(), i.getType(),i.getDocumentNum(), StockNumber);
									profitStockNumber =  profitStockNumber.add(StockNumber);;
									needNumber = 0;
									break;
								}
							}
							
							if(needNumber > 0){//说明还需要利预投
								//利预投，如果有资源则进行资源锁定
								List<Resource> orderList = ResourceControlCenter.searchResource(StockCode, "预投");
								for(Resource i : orderList ){
									BigDecimal orderNumber = i.getNumber();
									if(orderNumber.intValue() > needNumber){
										//说明资源数量比需求数量大
										ResourceControlCenter.lockResources(StockCode, id, "生产任务书", i.getID(), i.getType(),i.getDocumentNum(), new BigDecimal(needNumber));
										profitPreinvestment = profitPreinvestment.add(new BigDecimal(needNumber)) ;
										needNumber = 0;
										break;
									}else if(orderNumber.intValue() < needNumber){
										//说明资源数量比需求数量小
										ResourceControlCenter.lockResources(StockCode, id, "生产任务书", i.getID(), i.getType(),i.getDocumentNum(), orderNumber);
										profitPreinvestment = profitPreinvestment.add(new BigDecimal(needNumber)) ;
										needNumber -= orderNumber.intValue();
									}else{
										//说明资源数量与需求数量一样
										ResourceControlCenter.lockResources(StockCode, id, "生产任务书", i.getID(), i.getType(),i.getDocumentNum(), new BigDecimal(needNumber));
										profitPreinvestment = profitPreinvestment.add(orderNumber) ;
										needNumber = 0;
										break;
									}
								}
							}
						}
							
						BigDecimal requirement = planNum.subtract(profitStockNumber).subtract(profitPreinvestment);//最终需求量
						requirement = requirement.divide(attritionRate,2,BigDecimal.ROUND_HALF_UP);//除上工艺出品率（耗损率）
						requirement = requirement.setScale(2,BigDecimal.ROUND_HALF_UP);
						sql = "insert into MES_MRPResult  (fID,version,fTaskBookID,fStockCode,fRationNum,fPlanNumber,fParent,fParentNumber," +
								"fInventory,fPrecastNumber,fRequirement,fCreatePersonID,fCreateDateTime,fLastUpdatePersonID,fLastUpdateDateTime,fSource,fFinalNum) " +
								"values ('"+id+"',0,'"+BookID+"','"+StockCode+"',"+RationNum+","+planNum.setScale(2,BigDecimal.ROUND_HALF_UP)+",'"+parentID+"',"+parentNumber+","+profitStockNumber+","+profitPreinvestment+"," +
								""+requirement+",'"+personID+"',now(),'"+personID+"',now(),'"+Source+"',"+requirement+")";
						SQL.executeUpdate(Sql.DBMySql(sql), null, DATA_MODEL);
						MRPToBom(id,StockCode,requirement,MainPartNumber);
					}
					
				}
			}
		}catch(Exception e){
			e.printStackTrace();
			throw new RuntimeException();
		}
	}
	
	/**
	 * 
	 * @Title: createOrder
	 * @Description: 根据MRP运算的结果，生成每个零件的生产订单
	 * @param taskBookID	生产任务书ID
	 * @param type    生产任务书类别    预投 / 订单
	 * @return    参数
	 * @author ChenXiaoChen
	 * @date 2018-10-18
	 * @return JSONObject    返回类型
	 *
	 */
	public static JSONObject createOrder(String taskBookID,String type){
		/*
		 * 1.查出此生产任务书的MRP运算记录
		 * 2.找出所有需要生产的零件,然后根据运算记录展开，把此节点下所有订单都生成
		 * 3.根据运算结果，将生成的订单形成资源链路
		 */
		BookID = taskBookID;
		try{
			
			JSONObject result = new JSONObject();
			String sql = "select count(a.fid) as num from MES_MRPResult a where a.fTaskBookID = '"+taskBookID+"' and a.fFinalNum > 0 ";
			Table table = SQL.select(Sql.DBMySql(sql), null, DATA_MODEL);
			int num = table.iterator().next().getInt("num");//需要生成生产订单的条目数
			if(num == 0){//如果生成生产订单的条目数为0，说明此份生产任务书不需要生产
				ERRORSTATUS = "nothing";
			}
			sql = "select a.* from MES_MRPResult a where a.fTaskBookID = '"+taskBookID+"' and a.fDeliveryDate is not null and a.fFinalNum > 0 ";//将所有零件查出
			Table resultTable = SQL.select(Sql.DBMySql(sql), null, DATA_MODEL);
			Iterator<Row> rows = resultTable.iterator();    
			while(rows.hasNext()){
				Row row = rows.next();
				String currentID = row.getString("fID");//MRP运算记录表ID
				String StockCode = row.getString("fStockCode");//物料编码
				BigDecimal FinalNum = row.getDecimal("fFinalNum");//最终数
				String ListID = row.getString("fListID");//生产任务书明细ID
				Date endDate = row.getDate("fDeliveryDate");//交货期
				String CommonModeSign = row.getString("fCommonModeSign");//共模标识
				//生成生产订单
				makeOrder(currentID,StockCode,FinalNum,ListID,endDate,CommonModeSign,type);
				//生成资源链路
				sql = "select a.* from MES_ProductionOrder a where a.fMRPResultID = '"+currentID+"'";
				Table orderTable = SQL.select(Sql.DBMySql(sql), null, DATA_MODEL);
				String orderNum = orderTable.iterator().next().getString("fDocumentNumber");
				String orderID = orderTable.iterator().next().getString("fID");
				String TreeID = UUID.randomUUID().toString().replace("-", "");
				sql = "insert into MES_TreeOrder (fID,version,fRelevantID,fMaterialSource,fStockCode,fTaskBookID,fCreatePersonID,fCreateDateTime,fLastUpdatePersonID,fLastUpdateDateTime) " +
						"values('"+TreeID+"',0,'"+orderID+"','生产','"+StockCode+"','"+BookID+"','"+personID+"',now(),'"+personID+"',now())";
				SQL.executeUpdate(Sql.DBMySql(sql), null, DATA_MODEL);
				
				if("预投".equals(type) && endDate == null){
					//当前生产任务书需要锁定新生成的预投订单
					ResourceControlCenter.lockResources(StockCode, ListID, "生产任务书", orderID, "预投", orderNum, FinalNum);
				}else{
					//查找是否有锁定的资源，如果有，需要更新资源
					//List<Resource> list = ResourceControlCenter.findResouce(StockCode, needID, needType);
					
				//	ResourceControlCenter.updateResources(StockCode, ListID, "生产任务书", afterResourceID, afterResourcesType, afterResourceNumber, number);
				}
				
				makeTree(orderID,TreeID);
			}
			//将生产任务书的状态置为已运算
			sql = "update MES_BookOfManage a set a.fStatus = '已生成生产订单',a.version = version + 1 where a.fID = '"+taskBookID+"' ";
			SQL.executeUpdate(Sql.DBMySql(sql), null, DATA_MODEL);
			
			result.put("status", ERRORSTATUS);
			result.put("message", ERRORMESSAGE);
			return result;
		}catch(Exception e){
			e.printStackTrace();
			throw new RuntimeException();
		}
	}
	
	/**
	 * 
	 * @Title: makeOrder
	 * @Description: 根据MRP运算结果，递归生成生产订单，备料计划，工序计划，并计算出工序以及订单的标准生产周期
	 * @param MRPResultID			MRP运算结果ID
	 * @param Code					存货编码
	 * @param number				上级数量
	 * @param ListID				生产任务书明细ID
	 * @param endDate				交货期
	 * @param endDate    参数	
	 * @author ChenXiaoChen
	 * @date 2018-10-18
	 * @return void    返回类型
	 *
	 */
	public static void makeOrder(String MRPResultID,String Code,BigDecimal number,String ListID,Date endDate,String ComModeSign,String type){
		try{
			//生成生产订单
			if(ListID == null){
				ListID = "";
			}
			String orderType = "生产订单";
			if("预投".equals(type)) orderType = "预投订单";
			String ID = UUID.randomUUID().toString().replace("-", "");
			if(endDate == null){
				String ksql = "insert into MES_ProductionOrder a (a,a.version,a.fType,a.fSource,a.fDocumentNumber,a.fStockCode,a.fNumber,a.fOrderStatus," +
						"a.fTaskBookID,a.fTaskBookListID,a.fMRPResultID,a.fCreatePersonID,a.fCreateDateTime,a.fLastUpdatePersonID,a.fLastUpdateDateTime,a.fSafetyLead,a.fMouldCode) " +
						"values('"+ID+"',0,'"+orderType+"','MRP'," +
						":nextSeqString(format('%s%s','SC',format('%1$ty%1$tm%1$td', currentDateTime())),'0000')," +
						"'"+Code+"',"+number+",'计划','"+BookID+"','"+ListID+"','"+MRPResultID+"',:currentPersonID(),:currentDateTime(),:currentPersonID(),:currentDateTime(),0," +
						"(select c.fStockCode from CAPP_MouldProductList b optional join CAPP_Mould c on b.fMainID = c where b.fStockCode = '"+Code+"' and c.fStatus = '有效'))";
				KSQL.executeUpdate(ksql, null, DATA_MODEL, null);
			}else{
				String ksql = "insert into MES_ProductionOrder a (a,a.version,a.fType,a.fSource,a.fDocumentNumber,a.fStockCode,a.fNumber,a.fOrderStatus," +
						"a.fTaskBookID,a.fTaskBookListID,a.fMRPResultID,a.fCreatePersonID,a.fCreateDateTime,a.fLastUpdatePersonID,a.fLastUpdateDateTime,a.fSafetyLead,a.fDeliveryDate,a.fCommonModeSign,a.fMouldCode) " +
						"values('"+ID+"',0,'"+orderType+"','MRP'," +
						":nextSeqString(format('%s%s','SC',format('%1$ty%1$tm%1$td', currentDateTime())),'0000')," +
						"'"+Code+"',"+number+",'计划','"+BookID+"','"+ListID+"','"+MRPResultID+"',:currentPersonID(),:currentDateTime(),:currentPersonID(),:currentDateTime(),0,stringToDate('"+endDate+"'),'"+ComModeSign+"'," +
						"(select c.fStockCode from CAPP_MouldProductList b optional join CAPP_Mould c on b.fMainID = c where b.fStockCode = '"+Code+"' and c.fStatus = '有效'))";
				KSQL.executeUpdate(ksql, null, DATA_MODEL, null);
			}
			
			//生成备料计划
			String sql = "select a.* from MES_MRPResult a where a.fParent = '"+MRPResultID+"'";
			Table result = SQL.select(Sql.DBMySql(sql), null, DATA_MODEL);
			if(result.size() > 0){//如果有子节点，则生成备料计划
				Iterator<Row> rows = result.iterator(); 
				while(rows.hasNext()){
					Row row = rows.next();
					String currentID = row.getString("fID");//MRP运算记录表ID
					String StockCode = row.getString("fStockCode");//存货编码
					String Source = row.getString("fSource");//物料来源
					BigDecimal RationNum = row.getDecimal("fRationNum");//定额用量
					BigDecimal PlanNumber = row.getDecimal("fPlanNumber");//计划用量
					String CommonModeSign = row.getString("fCommonModeSign");//共模标识
					BigDecimal FinalNum = row.getDecimal("fFinalNum");//最终量
					String fListID = row.getString("fListID");//生产任务书明细ID
					sql = "insert into MES_StockPlan (fID,version,fMaterialSource,fMainID,fStockCode,fRationNum,fPlanNumber,fCreatePersonID,fCreateDateTime,fLastUpdatePersonID,fLastUpdateDateTime,fMRPResultID) " +
							"values(REPLACE(uuid(), '-',''),0,'"+Source+"','"+ID+"','"+StockCode+"',"+RationNum+","+FinalNum+",'"+personID+"',now(),'"+personID+"',now(),'"+currentID+"') ";
					SQL.executeUpdate(Sql.DBMySql(sql), null, DATA_MODEL);
					if("生产".equals(Source) && FinalNum.intValue() > 0){
						makeOrder(currentID,StockCode,FinalNum,fListID,null,CommonModeSign,type);
					}
//					if("配方".equals(Source)){//如果是配方，需要带出下面的配方清单
//						sql = "select a.* from MES_MRPResult a where a.fParent = '"+currentID+"'";
//						Table formulaList =  SQL.select(Sql.DBMySql(sql), null, DATA_MODEL);
//						Iterator<Row> formulaListRows = formulaList.iterator(); 
//						while(formulaListRows.hasNext()){
//							Row formulaListRow = formulaListRows.next();
//							StockCode = formulaListRow.getString("fStockCode");//物料编码
//							RationNum = formulaListRow.getDecimal("fRationNum");//定额用量
//							PlanNumber = formulaListRow.getDecimal("fPlanNumber");//计划用量
//							sql = "insert into MES_StockPlan (fID,version,fMaterialSource,fMainID,fStockCode,fRationNum,fPlanNumber,fCreatePersonID,fCreateDateTime,fLastUpdatePersonID,fLastUpdateDateTime,fMRPResultID) " +
//									"values(REPLACE(uuid(), '-',''),0,'采购','"+ID+"','"+StockCode+"',"+RationNum+","+PlanNumber+",'"+personID+"',now(),'"+personID+"',now(),'"+currentID+"') ";
//							SQL.executeUpdate(Sql.DBMySql(sql), null, DATA_MODEL);
//						}
//					}
				}
			}
			//查找对应工艺路线，生成工序计划
			sql = "insert into MES_ProcedurePlan (fID,version,fMainID,fCode,fName,fWorkCenterID,fEachGroupNumber,fSingleGroupHour," +
					"fQuasiJunctions,fProcessCycle,fInToNumber,fFinishNumber,fOutToNumber,fResourcesNumber,fClassesNumber,fTotalHours," +
					"fCreatePersonID,fCreateDateTime,fLastUpdatePersonID,fLastUpdateDateTime,fStandardCycle,fIsFinish) select REPLACE(uuid(), '-',''),0,'"+ID+"',a.fCode," +
					"a.fName,a.fWorkCenterID,a.fEachGroupNumber,a.fSingleGroupHour,a.fQuasiJunctions,a.fProcessCycle,0,0,0,1," +
					"1,(("+number+"/a.fEachGroupNumber) * a.fSingleGroupHour + a.fQuasiJunctions)/60,'"+personID+"',now(),'"+personID+"',now()," +
					" CEILING((("+number+"/a.fEachGroupNumber) * a.fSingleGroupHour + a.fQuasiJunctions)/60/8 + (a.fProcessCycle/24) + (select c.fBeforeDay + c.fAfterDay from BAS_WorkCenter c where c.fID =a.fWorkCenterID )),'否' " +
					"from CAPP_Procedure a left join CAPP_Technology b on a.fMainID = b.fID where b.fStockCode = '"+Code+"' and b.fStatus = '有效'";
			SQL.executeUpdate(Sql.DBMySql(sql), null, DATA_MODEL );
			
			//更新订单的标准生产周期
			sql = "update MES_ProductionOrder a set a.fStandardCycle = (select sum(b.fStandardCycle) from MES_ProcedurePlan b where b.fMainID = a.fID) where a.fID = '"+ID+"' ";
			SQL.executeUpdate(Sql.DBMySql(sql), null, DATA_MODEL);
		}catch(Exception e){
			e.printStackTrace();
			throw new RuntimeException();
		}
	}
	
	/**
	 * 
	 * @Title: makeTree
	 * @Description: 	将生成的生产订单构建订单结构树
	 * @param orderID	生产订单ID
	 * @param parentID  父ID
	 * @author ChenXiaoChen
	 * @date 2018-10-18
	 * @return void    返回类型
	 *
	 */
	public static void makeTree(String orderID,String parentID){
		try{
			String sql = "select a.* from MES_StockPlan a where a.fMainID = '"+orderID+"'";
			Table stockPlanTable = SQL.select(Sql.DBMySql(sql), null, DATA_MODEL);//查询此订单下的备料计划
			Iterator<Row> rows = stockPlanTable.iterator(); 
			while(rows.hasNext()){
				Row row = rows.next();
				String currentID = row.getString("fID");
				String MaterialSource = row.getString("fMaterialSource");//物料来源
				String StockCode = row.getString("fStockCode");//存货编码
				String MRPResultID = row.getString("fMRPResultID");//MRP运算表ID，记录此备料计划来源于哪条运算记录
				String RelevantID = "";//相关ID，当类型为生产时，查找是否有对应的生产订单，如果有，则存生产订单ID，反之存备料计划ID，
																					//类型为采购和配方存的为备料计划ID
				boolean flag = false;//用于判断此条备料计划是否生成了订单
				if("生产".equals(MaterialSource)){
					sql = "select a.fID from MES_ProductionOrder a where a.fMRPResultID = '"+MRPResultID+"' ";
					Table order = SQL.select(Sql.DBMySql(sql), null, DATA_MODEL);
					if(order.size() > 0){
						flag = true;
						RelevantID = order.iterator().next().getString("fID");//生产订单ID
						String orderNo = order.iterator().next().getString("fDocumentNumber");//单据编号
						int number = order.iterator().next().getInt("fNumber");//订单数量
						//将子件锁定给上层订单
						ResourceControlCenter.lockResources(StockCode, orderID, "备料计划", RelevantID, "生产订单", orderNo, new BigDecimal(number));
					}
					
				}else{
					RelevantID = currentID;
				}
				String ID = UUID.randomUUID().toString().replace("-", "");
				sql = "insert into MES_TreeOrder (fID,version,fRelevantID,fMaterialSource,fStockCode,fParent,fTaskBookID," +
						"fCreatePersonID,fCreateDateTime,fLastUpdatePersonID,fLastUpdateDateTime) " +
						"values('"+ID+"',0,'"+RelevantID+"','"+MaterialSource+"','"+StockCode+"','"+parentID+"','"+BookID+"','"+personID+"',now(),'"+personID+"',now())";
				SQL.executeUpdate(Sql.DBMySql(sql), null, DATA_MODEL);
				if("生产".equals(MaterialSource) && flag){
					makeTree(RelevantID,ID);//递归将此零件下的订单生成订单结构树
				}
			}
		}catch(Exception e){
			e.printStackTrace();
			throw new RuntimeException();
		}
	}
	
	/**
	 * 
	 * @Title: MPSOperation
	 * @Description: TODO(MPS运算)
	 * @param startDate    开始日期
	 * @param endDate      结束日期
	 * @param autoMatchingMould    自动匹配模具使用性
	 * @param isConsiderOrder      是否考虑在制订单
	 * @param isConsiderStock      是否考虑库存
	 * 参数
	 * @author ChenXiaoChen
	 * @date 2019-4-10
	 * @return void    返回类型
	 *
	 */
	private static int prospectPeriodDay = 0;//展望期
	private static String ID = "";//
	private static Date startDate = null;//展望期开始时间
	private static Date endDate = null;//展望期结束时间
	private static String message = "";
	
	public static String MPSOperation(Date startDate,Date endDate,Integer autoMatchingMould,Integer isConsiderOrder,Integer isConsiderStock){
		/**
		 * MPS计算步骤
		 * 1.核销上一次MPS的运算结果
		 * 2.生成本次运算的MPS运算单
		 * 3.找出MPS件，对每个可以运算的MPS件进行运算
		 * 4.对每个零件的运算过程进行记录
		 * 5.根据运算结果，对于需要投产的零件进行记录
		 */
		//计算展望期
		try {
			prospectPeriodDay = tool.daysBetween(startDate, endDate);
			Calculation.startDate = startDate;
			Calculation.endDate = endDate;
			
			//将之前计算的结果全部作废掉
			String sql = "update MES_MPSResult a set a.fStatus = '已作废' where a.fStatus = '已运算' ";
			SQL.executeUpdate(sql, null, DATA_MODEL);
			
			//生成本次运算单
			ID = UUID.randomUUID().toString().replace("-", "");//32位字符串
			sql = "insert into MES_MPSResult a (a,a.version,a.fDocumentNumber,a.fIsConsiderStock,a.fIsConsiderOrder," +
					"a.fProspectPeriod,a.fStatus,a.fCreatePersonID,a.fCreateDateTime,a.fLastUpdatePersonID,a.fLastUpdateDateTime) " +
					"values('"+ID+"',0,:nextSeqString(format('%s%s','MPS',format('%1$ty%1$tm%1$td', :currentDateTime())),'0000')," +
					""+isConsiderStock+","+isConsiderOrder+",'"+startDate+"~"+endDate+"','已运算',:currentPersonID(),:currentDateTime()," +
					":currentPersonID(),:currentDateTime()) ";
			KSQL.executeUpdate(sql, null, DATA_MODEL, null);
			
			//找出需要计算的MPS件
			sql = "select a.fStockCode,a.fStockName,a.fNormsType,a.fMaterialType,a.fCustomerCode,ifnull(a.fAddUpLeadTime,-1) as fAddUpLeadTime," +
					"ifnull(a.fSafetyLead,-1) as fSafetyLead,ifnull(a.fSafetyStock,-1) as fSafetyStock,ifnull(a.fEconomicBatch,-1) as fEconomicBatch," +
					"ifnull(a.fFitOutLeadTime,-1) as fFitOutLeadTime,ifnull(a.fBatchIncrement,-1) as fBatchIncrement  " +
					"from BAS_StockArchives a where a.fIsMps = '是' ";
			Table mpsTable = SQL.select(Sql.DBMySql(sql), null, DATA_MODEL);
			Iterator<Row> rows = mpsTable.iterator(); 
			while(rows.hasNext()){
				//对每个MPS件进行计算
				String currentMessage = "";
				Row row = rows.next();
				String code = row.getString("fStockCode");//存货编码
				String name = row.getString("fStockName");//存货名称
				String normsType = row.getString("fNormsType");//规格型号
				String materialType = row.getString("fMaterialType");//材质类型
				String customerCode = row.getString("fCustomerCode");//客服商品代码
				int addUpLeadTime = row.getInt("fAddUpLeadTime");//累计提前期
				int safetyLead = row.getInt("fSafetyLead");//安全提前期
				int safetyStock = row.getInt("fSafetyStock");//安全库存
				int economicBatch = row.getInt("fEconomicBatch");//经济批量
				int fitOutLeadTime = row.getInt("fFitOutLeadTime");//总装提前期
				int batchIncrement = row.getInt("fBatchIncrement");//批量增量
				if(MPSParamterCheck(addUpLeadTime)) currentMessage += code + "/" + name + " 累计提前期参数为空";
				if(MPSParamterCheck(safetyLead)) currentMessage += " 安全提前期参数为空";
				if(MPSParamterCheck(safetyStock)) currentMessage += " 安全库存参数为空";
				if(MPSParamterCheck(economicBatch)) currentMessage += " 经济批量参数为空";
				if(MPSParamterCheck(fitOutLeadTime)) currentMessage += " 总装提前期参数为空";
				if(currentMessage.length() > 0){
					message += currentMessage + "\n";
					continue;
				}
//				Materiel materiel = new Materiel(code, name, normsType, materialType, customerCode, addUpLeadTime, safetyLead, safetyStock, economicBatch, fitOutLeadTime, batchIncrement);
				
				Materiel materiel = new Materiel(code, name, normsType, materialType, customerCode, addUpLeadTime, safetyLead, safetyStock, economicBatch, fitOutLeadTime, batchIncrement);
				MPSOperationProcess(materiel, autoMatchingMould, isConsiderOrder, isConsiderStock);
			}
			if(message.length() > 0){
				sql = "update MES_MPSResult a set a.fResult = '"+message+"' where a.fid = '"+ID+"' ";
				SQL.executeUpdate(Sql.DBMySql(sql), null, DATA_MODEL);
			}
		} catch (ParseException e) {
			e.printStackTrace();
			throw new RuntimeException(e.getMessage());
		}
		return "yes";
	}
	
	public static boolean MPSParamterCheck(int p){
		if(p < 0)
			return true;
		return false;	
	}
	
	/**
	 * 
	 * @Title: MPSOperationProcess
	 * @Description: TODO(对单个物料进行运算)
	 * @param code			存货编码
	 * @param addUpLeadTime	累计提前期
	 * @param safetyLead		安全提前期
	 * @param safetyStock	安全库存
	 * @param economicBatch	经济批量
	 * @param fitOutLeadTime	总装提前期
	 * @param batchIncrement 批量增量
	 * 参数
	 * @author ChenXiaoChen
	 * @date 2019-4-11
	 * @return void    返回类型
	 *
	 */
	public static void MPSOperationProcess(Materiel materiel,Integer autoMatchingMould,Integer isConsiderOrder,Integer isConsiderStock){
		/*
		 * MPS运算过程
		 * 1.准备工作（初始化记录表，算出3个时区的长度）
		 * 2.进行循环运算每天的各个值
		 * 3.将结果写入数据库
		 */
		
		//1.准备工作（初始化记录表，算出3个时区的长度）--------------------------------
		int process[][] = new int[13][prospectPeriodDay + 1];//当前物料的运算记录表     下标都从1开始使用
		
		//各个时区的含义
		//需求时区           需求时区 = 总装提前期
		//计划时区		计划时区 = 累计提前期  - 总装提前期
		//预测时区		预测时区 = 展望期 - 累计提前期
		
		//根据展望期和参数计算出3个时区的节点
		
		int demandTimeZone = materiel.getFitOutLeadTime();//需求时区节点         需求时区节点 = 总装提前期
		int plannedTimeZone =  materiel.getAddUpLeadTime();//计划时区节点		计划时区节点 = 累计提前期  
		int forecastTimeZone = prospectPeriodDay ;//预测时区节点		预测时区 = 展望期天数
		int result = initializationProcess(process,materiel.getStockCode(),demandTimeZone, plannedTimeZone, forecastTimeZone,isConsiderOrder);//对数组的需求进行初始化
		if(result == 0 && materiel.getSafetyStock() == 0){//说明订单没有需求
			return;
		}
		
		
		
		//2.进行循环运算每天的各个值
		concreteCalculation(process,materiel, autoMatchingMould,  isConsiderOrder, isConsiderStock);
		
		//3.写入数据库
		insertDataBase(process,materiel,demandTimeZone,plannedTimeZone,forecastTimeZone);
		
		
	}
	
	public static int initializationProcess(int process[][],String code,int demandTimeZone,int plannedTimeZone,int forecastTimeZone,int isConsiderOrder ){		
		//行为参数值，列为日期	行：1.预测量	
//									2.订单量		
//									3.毛需求量	
//									4.计划接收量		
//									5.PAB初值	
//									6.净需求量
//									7.计划产出量
//									8.PAB
//									9.计划投入量
//									10.ATP	
//									11.交货期
//									12.标识        记录是否压缩
		for(int i = 0;i <= 12;i++)
			for(int j = 0;j<prospectPeriodDay;j++)
				process[i][j] = 0;
		
		Calendar calendar = Calendar.getInstance();//创建日历对象
		calendar.setTime(new Date());
		//找出展望期内的所有此零件的投产订单
		String sql = "select c.fStockCode,a.FDate,sum(a.FAuxQty) as AuxQty from SEOrderEntry a left join SEOrder b on b.FInterID = a.FInterID " +
				"left join BAS_StockArchives c on c.fItemID = a.FItemID " +
				"left join MES_ResourceAllocation d on d.fNeedID = a.FDetailID and d.fIsClose = '否' " +
				"where c.fStockCode = '"+code+"' and b.FClosed = 0 and a.FDate between '"+startDate+"' and '"+endDate+"' " +  //
				"group by c.fStockCode,a.FDate " +
				"having (ifnull(sum(a.FAuxQty),0) - ifnull(sum(a.FCommitQty),0) - ifnull(sum(d.fNumber),0) ) > 0  order by a.FDate asc ";
		Table table = SQL.select(Sql.DBMySql(sql), null, DATA_MODEL);
		if(table.size() > 0){//说明有订单量
			int count = table.size();//数据的行数
			Iterator<Row> rows = table.iterator(); 
			Row row = rows.next();
			for(int i = 1,j = 1 ;i <= prospectPeriodDay ; i++){
				
				Date deliveryDate = row.getDateTime("FDate");
				if(tool.formatDate(calendar.getTime(), "yyyy-MM-dd").equals( tool.formatDate(deliveryDate, "yyyy-MM-dd"))){//说明日期相同,将统计的值写入
					process[2][i] = row.getDecimal("AuxQty").intValue();
					j++;
					if(j > count){	
						break;
					}
					row = rows.next();
				}
				
				if(isConsiderOrder == 1){
					//如果考虑在制订单
					sql = "select a.fID,a.fStockCode,a.fNumber,sum(b.fNumber),(a.fNumber - ifnull(sum(b.fNumber),0)) as lastNumber " +
							"from MES_ProductionOrder a " +
							"left join MES_ResourceAllocation b on b.fResourceID = a.fID and b.fIsClose = '否' " +
							" where a.fStockCode = '"+code+"' and a.fPlannedEndDate = '"+tool.formatDate(calendar.getTime(), "yyyy-MM-dd")+"' " +
							"group by a.fID,a.fStockCode,a.fNumber " +
							"having a.fNumber > ifnull(sum(b.fNumber),0) ";
					Table orderTable = SQL.select(sql, null, DATA_MODEL);
					if(orderTable.size() > 0){
						//当有订单资源，则记录为计划接收量
						int receiveNumber = orderTable.iterator().next().getDecimal("lastNumber").intValue();//剩余数量
						process[i][4] = receiveNumber;
					}
				}
				
				
				calendar.add(Calendar.DAY_OF_YEAR, 1);
			}
			
			//根据不用的时区,算出毛需求量
			//在需求时区,毛需求等于订单量
			//在计划时区,毛需求等于预测量和订单量中的较大值
			//在预测时区,毛需求等于预测量
			for(int i = 1;i <= prospectPeriodDay ;i++){
				if(i <= demandTimeZone){
					process[3][i] = process[2][i];
				}else if(i <= plannedTimeZone){
					process[3][i] = process[2][i] > process[1][i]? process[2][i] : process[1][i] ;
				}else if(i <= forecastTimeZone){
					process[3][i] = process[1][i];
				}
			}
		}else{
			return 0;//代表此零件没有需求
		}
		return 1;
	}
	
	public static void concreteCalculation(int process[][],Materiel materiel, int autoMatchingMould, int isConsiderOrder,int isConsiderStock){
		int stock = 0;//当前库存
		if(isConsiderStock == 1){//说明需要考虑库存数据
			//读取库存数据,待完善
			List<Resource> list = ResourceControlCenter.searchResource(materiel.getStockCode(), "库存");
			for(Resource r : list){
				stock += r.getNumber().intValue();
			}
		}
		String resultID = UUID.randomUUID().toString().replace("-", "");//32位字符串
		String sql = "insert into MES_MPSResultList (fid,version,fMainID,fStockCode,fAddUpLeadTime,fSafetyLead,fSafetyStock," +
				"fEconomicBatch,fFitOutLeadTime,fBatchIncrement,fSign,fCurrentInventory) values('"+resultID+"',0,'"+ID+"','"+materiel.getStockCode()+"'," +
				" "+materiel.getAddUpLeadTime()+","+materiel.getSafetyLead()+","+materiel.getSafetyStock()+","+materiel.getEconomicBatch()+","+materiel.getFitOutLeadTime()+","+materiel.getBatchIncrement()+",0,"+stock+")";
		SQL.executeUpdate(Sql.DBMySql(sql), null, DATA_MODEL);
		
		process[8][0] = stock;
		for(int i = 1 ;i <= prospectPeriodDay ;i++){
			//PAB初值     PAB初值  = 上一个时间的 PAB值 - 毛需求量
			process[5][i] = process[8][i-1] - process[3][i];
			//净需求量    净需求量 = 本时段毛需求量 - 前一时段末的PAB - 本时段的计划接收量 + 安全库存量
			process[6][i] = process[3][i] - process[8][i-1] - process[4][i] + materiel.getSafetyStock();
//			if(process[5][i] > materiel.getSafetyStock()){
//				process[6][i] = 0;
//			}else{
//				process[6][i] = materiel.getSafetyStock() - process[5][i];
//			}
			
			if(process[6][i] > 0){//如果净需求量大于0 ，就需要补充
				int dValue = materiel.getSafetyStock() - process[5][i];//与安全库存相差得数量
				int batchNum = dValue / materiel.getEconomicBatch();//经济批量的倍数
				if(batchNum == 0){
					batchNum++;
				}
				//计划产出量         计划产出量 = 经济批量 * 经济批量的倍数   
				process[7][i] = batchNum * materiel.getEconomicBatch() ;
			}else{//反之则不需要产出
				process[7][i] = 0;//计划产出量
			}
			
			
			if(process[7][i] > 0){//如果计划产出量大于0 ，说明需要投产，根据累计提前期和安全提前期，算出投产时间
				//将当前时间的计划产出量按累计提前期往前推，计算出各个时间段的计划投入量
				if( (i - materiel.getAddUpLeadTime()) < 0 ){
					process[9][1] = process[9][1] + process[7][i];//计划投入量
					if(process[11][1] > i ){
						process[11][1] = i;
						process[12][1] = 1;
					}
				}else{
					process[9][i-materiel.getAddUpLeadTime()] = process[9][i-materiel.getAddUpLeadTime()] + process[6][i];//计划投入量
				}
			}
			
			//PAB值   PAB = 前一时段末的PAB+本时段计划接收量-本时段毛需求量+本时段计划产出量
			process[8][i] = process[8][i-1] + process[4][i] - process[3][i] + process[7][i];
		}
	}
	
	public static void insertDataBase(int process[][],Materiel materiel,int demandTimeZone,int plannedTimeZone,int forecastTimeZone){
		Calendar calendar = Calendar.getInstance();
		Calendar deliveryCalendar = Calendar.getInstance();
		calendar.setTime(startDate);
		for(int i = 1;i <= prospectPeriodDay;i++){
			deliveryCalendar.setTime(calendar.getTime());
			String processID = UUID.randomUUID().toString().replace("-", "");
			String timeZone = "";//时区
			if(i == 1){
				timeZone = "当前时间";
			}else if(i <= demandTimeZone){
				timeZone = "需求时区";
			}else if (i <= plannedTimeZone){
				timeZone = "计划时区";
			}else if(i <= forecastTimeZone){
				timeZone = "预测时区";
			}
			
			String sql = "insert into MES_MPSProcessResult (fid,version,fMainID,fStockCode,fTimeZone,fDate,fForecastNum," +
					"fOrderNum,fGrossDemand,fPlanReceive,fPlanProduceNum,fPlanPutNum,fDemand,fPAB) " +
					"values('"+processID+"',0,'"+ID+"','"+materiel.getStockCode()+"','"+timeZone+"','"+tool.formatDate(calendar.getTime(), "yyyy-MM-dd")+"'," +
					""+process[1][i]+","+process[2][i]+","+process[3][i]+","+process[4][i]+","+process[7][i]+","+process[9][i]+","+process[6][i]+","+process[8][i]+")";
			SQL.executeUpdate(Sql.DBMySql(sql), null, DATA_MODEL);
			if(process[9][i] > 0){//如果计划投入量大于0，说明当前日期需要投产
				String listID = UUID.randomUUID().toString().replace("-", "");
				deliveryCalendar.add(Calendar.DAY_OF_YEAR, process[12][i]);
				sql = "insert into MES_MPSResultList (fid,version,fMainID,fStockCode,fDate,fDeliveryDate,fNumber," +
						"fAddUpLeadTime,fSafetyLead,fSafetyStock,fFitOutLeadTime,fEconomicBatch,fBatchIncrement,fSign,fStatus) " +
						"values('"+listID+"',0,'"+ID+"','"+materiel.getStockCode()+"','"+tool.formatDate(calendar.getTime(), "yyyy-MM-dd")+"'," +
						" '"+tool.formatDate(deliveryCalendar.getTime(), "yyyy-MM-dd")+"',"+process[9][i]+","+materiel.getAddUpLeadTime()+","+materiel.getSafetyLead()+"," +
						" "+materiel.getSafetyStock()+","+materiel.getFitOutLeadTime()+","+materiel.getEconomicBatch()+","+materiel.getBatchIncrement()+"," +
						" 1,'未投产')";
				SQL.executeUpdate(Sql.DBMySql(sql), null, DATA_MODEL);
			}
			calendar.add(Calendar.DAY_OF_YEAR, 1);
		}
	}
	
	/**
	 * 
	 * @Title: resourceComputing
	 * @Description: TODO(对客户订单的需求按需求时间进行重新匹配资源)
	 * @    参数
	 * @author ChenXiaoChen
	 * @date 2019-7-2
	 * @return void    返回类型
	 *
	 */
	public static String resourceComputing(){
		/**
		 * 对客户订单的需求按需求时间进行重新匹配资源，按交货期的先后顺序进行升序排序
		 * 依次分配资源，资源的现后顺序按  库存、生产订单、生产任务书 顺序进行分配
		 */
		try{
			//对需求为客户订单的资源分配全部释放
			String sql = "select a.fStockCode,a.fNeedID,a.fNeedType from MES_ResourceAllocation a " +
					"where a.fNeedType = '客户订单' and a.fIsClose = '否' " +
					"group by a.fStockCode,a.fNeedID,a.fNeedType ";
			Table reTable = SQL.select(sql, null, DATA_MODEL);
			if(reTable.size() > 0){
				Iterator<Row> rows = reTable.iterator(); 
				while(rows.hasNext()){
					Row row = rows.next();
					String code = row.getString("fStockCode");//物料编码
					String needID = row.getString("fNeedID");//需求ID
					String needType = row.getString("fNeedType");//需求类别
					ResourceControlCenter.unlockAllResourcesOfDemand(code, needID, needType);
				}
			}
			
			//根据客户的需求日期，按升序进行排序，依次分配资源
			sql = "select a.*, (a.FAuxQty - a.FCommitQty) as lastNumber ,c.fStockCode,b.FBillNo from SEOrderEntry a " +
					"left join SEOrder b on b.finterid = a.finterid " +
					"left join BAS_StockArchives c on a.FItemID = c.fItemID " +
					"where a.FAuxQty > a.FCommitQty and b.FClosed = 0 " +
					"order by a.FDate asc ";
			Table seOrderTable = SQL.select(sql, null, DATA_MODEL);
			if(seOrderTable.size() > 0){
				Iterator<Row> rows = seOrderTable.iterator(); 
				while(rows.hasNext()){
					Row row = rows.next();
					String needID = row.getInt("FDetailID")+"";//需求ID
					String code = row.getString("fStockCode");//编码
					int needNumber = row.getDecimal("lastNumber").intValue();//剩余数量
					List<Resource> list = ResourceControlCenter.searchAllResouce(code);
					for(Resource r : list){
						int reNumber = r.getNumber().intValue();//获取资源数量
						int lockNumber = 0;
						if(reNumber > needNumber) lockNumber = needNumber;  //如果资源数量大于需求数量，则只分配需求数量
						else if(reNumber <= needNumber) lockNumber = reNumber;  //如果资源数量小于需求数量，则分配资源数量
						//对资源进行锁定
						ResourceControlCenter.lockResources(code, needID, "客户订单", r.getID(), r.getType(), r.getDocumentNum(), new BigDecimal(lockNumber));
						needNumber -= lockNumber;
						if(needNumber == 0) break;
					}
				}
			}	
			return "success";
		}catch(Exception e){
			e.printStackTrace();
			throw new RuntimeException(e.getMessage());
		}
		
	}
}
